@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 
 This file is part of qpdfview.
@@ -94,28 +95,6 @@ void BookmarkMenu::removeJumpToPageAction(int page)
     }
 }
 
-QString BookmarkMenu::absoluteFilePath() const
-{
-    return menuAction()->data().toString();
-}
-
-JumpList BookmarkMenu::jumps() const
-{
-    JumpList jumps;
-
-    foreach(const QAction* action, m_jumpToPageActionGroup->actions())
-    {
-        jumps.append(qMakePair(action->data().toInt(), action->text()));
-    }
-
-    return jumps;
-}
-
-void BookmarkMenu::on_removeBookmark_triggered()
-{
-    deleteLater();
-}
-
 void BookmarkMenu::on_open_triggered()
 {
     emit openTriggered(absoluteFilePath());
@@ -131,4 +110,9 @@ void BookmarkMenu::on_jumpToPage_triggered(QAction* action)
     emit jumpToPageTriggered(absoluteFilePath(), action->data().toInt());
 }
 
+void BookmarkMenu::on_removeBookmark_triggered()
+{
+    emit removeBookmarkTriggered(absoluteFilePath());
+}
+
 } // qpdfview
