@@ -33,11 +33,47 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <libdjvu/ddjvuapi.h>
 #include <libdjvu/miniexp.h>
 
+#define LOCK_PAGE QMutexLocker mutexLocker(&m_parent->m_mutex);
+#define LOCK_DOCUMENT QMutexLocker mutexLocker(&m_mutex);
+
+#if DDJVUAPI_VERSION < 23
+
+#define LOCK_PAGE_GLOBAL QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+#define LOCK_DOCUMENT_GLOBAL QMutexLocker globalMutexLocker(m_globalMutex);
+
+#else
+
+#define LOCK_PAGE_GLOBAL
+#define LOCK_DOCUMENT_GLOBAL
+
+#endif // DDJVUAPI_VERSION
+
 namespace
 {
 
+using namespace qpdfview;
 using namespace qpdfview::Model;
 
+inline miniexp_t miniexp_cadddr(miniexp_t exp)
+{
+    return miniexp_cadr(miniexp_cddr(exp));
+}
+
+inline miniexp_t miniexp_caddddr(miniexp_t exp)
+{
+    return miniexp_caddr(miniexp_cddr(exp));
+}
+
+inline miniexp_t skip(miniexp_t exp, int offset)
+{
+    while(offset-- > 0)
+    {
+        exp = miniexp_cdr(exp);
+    }
+
+    return exp;
+}
+
 void clearMessageQueue(ddjvu_context_t* context, bool wait)
 {
     if(wait)
@@ -82,17 +118,16 @@ void waitForMessageTag(ddjvu_context_t* context, ddjvu_message_tag_t tag)
     }
 }
 
-QPainterPath loadLinkBoundary(const QString& type, miniexp_t linkExp, const QSizeF& size)
+QPainterPath loadLinkBoundary(const QString& type, miniexp_t boundaryExp, const QSizeF& size)
 {
     QPainterPath boundary;
 
-    miniexp_t areaExp = miniexp_nth(3, linkExp);
-    const int areaLength = miniexp_length(areaExp);
+    const int count = miniexp_length(boundaryExp);
 
-    if(areaLength == 5 && (type == QLatin1String("rect") || type == QLatin1String("oval")))
+    if(count == 4 && (type == QLatin1String("rect") || type == QLatin1String("oval")))
     {
-        QPoint p(miniexp_to_int(miniexp_nth(1, areaExp)), miniexp_to_int(miniexp_nth(2, areaExp)));
-        QSize s(miniexp_to_int(miniexp_nth(3, areaExp)), miniexp_to_int(miniexp_nth(4, areaExp)));
+        QPoint p(miniexp_to_int(miniexp_car(boundaryExp)), miniexp_to_int(miniexp_cadr(boundaryExp)));
+        QSize s(miniexp_to_int(miniexp_caddr(boundaryExp)), miniexp_to_int(miniexp_cadddr(boundaryExp)));
 
         p.setY(size.height() - s.height() - p.y());
 
@@ -107,13 +142,13 @@ QPainterPath loadLinkBoundary(const QString& type, miniexp_t linkExp, const QSiz
             boundary.addEllipse(r);
         }
     }
-    else if(areaLength > 0 && areaLength % 2 == 1 && type == QLatin1String("poly"))
+    else if(count > 0 && count % 2 == 0 && type == QLatin1String("poly"))
     {
         QPolygon polygon;
 
-        for(int areaExpN = 1; areaExpN < areaLength; areaExpN += 2)
+        for(int index = 0; index < count; index += 2)
         {
-            QPoint p(miniexp_to_int(miniexp_nth(areaExpN, areaExp)), miniexp_to_int(miniexp_nth(areaExpN + 1, areaExp)));
+            QPoint p(miniexp_to_int(miniexp_nth(index, boundaryExp)), miniexp_to_int(miniexp_nth(index + 1, boundaryExp)));
 
             p.setY(size.height() - p.y());
 
@@ -126,19 +161,17 @@ QPainterPath loadLinkBoundary(const QString& type, miniexp_t linkExp, const QSiz
     return QTransform::fromScale(1.0 / size.width(), 1.0 / size.height()).map(boundary);
 }
 
-Link* loadLinkTarget(const QPainterPath& boundary, miniexp_t linkExp, int index, const QHash< QString, int >& indexByName)
+Link* loadLinkTarget(const QPainterPath& boundary, miniexp_t targetExp, int index, const QHash< QString, int >& indexByName)
 {
     QString target;
 
-    miniexp_t targetExp = miniexp_nth(1, linkExp);
-
     if(miniexp_stringp(targetExp))
     {
-        target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, linkExp)));
+        target = QString::fromUtf8(miniexp_to_str(targetExp));
     }
-    else if(miniexp_length(targetExp) == 3 && qstrncmp(miniexp_to_name(miniexp_nth(0, targetExp)), "url", 3) == 0)
+    else if(miniexp_length(targetExp) == 3 && qstrcmp(miniexp_to_name(miniexp_car(targetExp)), "url") == 0)
     {
-        target = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, targetExp)));
+        target = QString::fromUtf8(miniexp_to_str(miniexp_cadr(targetExp)));
     }
 
     if(target.isEmpty())
@@ -166,7 +199,10 @@ Link* loadLinkTarget(const QPainterPath& boundary, miniexp_t linkExp, int index,
         }
         else
         {
-            targetPage = (target.at(0) == QLatin1Char('+') || target.at(0) == QLatin1Char('-')) ? index + targetPage : targetPage;
+            if(target.at(0) == QLatin1Char('+') || target.at(0) == QLatin1Char('-'))
+            {
+                targetPage += index + 1;
+            }
         }
 
         return new Link(boundary, targetPage);
@@ -177,30 +213,36 @@ Link* loadLinkTarget(const QPainterPath& boundary, miniexp_t linkExp, int index,
     }
 }
 
-QList< Link* > loadLinks(miniexp_t pageAnnoExp, const QSizeF& size, int index, const QHash< QString, int >& indexByName)
+QList< Link* > loadLinks(miniexp_t linkExp, const QSizeF& size, int index, const QHash< QString, int >& indexByName)
 {
     QList< Link* > links;
 
-    const int pageAnnoLength = miniexp_length(pageAnnoExp);
-
-    for(int pageAnnoN = 0; pageAnnoN < pageAnnoLength; ++pageAnnoN)
+    for(miniexp_t linkItem = miniexp_nil; miniexp_consp(linkExp); linkExp = miniexp_cdr(linkExp))
     {
-        miniexp_t linkExp = miniexp_nth(pageAnnoN, pageAnnoExp);
+        linkItem = miniexp_car(linkExp);
 
-        if(miniexp_length(linkExp) <= 3 || qstrncmp(miniexp_to_name(miniexp_nth(0, linkExp)), "maparea", 7 ) != 0 || !miniexp_symbolp(miniexp_nth(0, miniexp_nth(3, linkExp))))
+        if(miniexp_length(linkItem) < 4 || qstrcmp(miniexp_to_name(miniexp_car(linkItem)), "maparea") != 0)
         {
             continue;
         }
 
-        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_nth(0, miniexp_nth(3, linkExp))));
+        miniexp_t targetExp = miniexp_cadr(linkItem);
+        miniexp_t boundaryExp = miniexp_cadddr(linkItem);
+
+        if(!miniexp_symbolp(miniexp_car(boundaryExp)))
+        {
+            continue;
+        }
+
+        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_car(boundaryExp)));
 
         if(type == QLatin1String("rect") || type == QLatin1String("oval") || type == QLatin1String("poly"))
         {
-            QPainterPath boundary = loadLinkBoundary(type, linkExp, size);
+            QPainterPath boundary = loadLinkBoundary(type, miniexp_cdr(boundaryExp), size);
 
             if(!boundary.isEmpty())
             {
-                Link* link = loadLinkTarget(boundary, linkExp, index, indexByName);
+                Link* link = loadLinkTarget(boundary, targetExp, index, indexByName);
 
                 if(link != 0)
                 {
@@ -213,63 +255,155 @@ QList< Link* > loadLinks(miniexp_t pageAnnoExp, const QSizeF& size, int index, c
     return links;
 }
 
-QString loadText(miniexp_t textExp, const QRect& rect, int pageHeight)
+QString loadText(miniexp_t textExp, const QSizeF& size, const QRectF& rect)
 {
-    const int textLength = miniexp_length(textExp);
-
-    if(textLength >= 6 && miniexp_symbolp(miniexp_nth(0, textExp)))
+    if(miniexp_length(textExp) < 6 && !miniexp_symbolp(miniexp_car(textExp)))
     {
-        const int xmin = miniexp_to_int(miniexp_nth(1, textExp));
-        const int ymin = miniexp_to_int(miniexp_nth(2, textExp));
-        const int xmax = miniexp_to_int(miniexp_nth(3, textExp));
-        const int ymax = miniexp_to_int(miniexp_nth(4, textExp));
+        return QString();
+    }
 
-        if(rect.intersects(QRect(xmin, pageHeight - ymax, xmax - xmin, ymax - ymin)))
+    const int xmin = miniexp_to_int(miniexp_cadr(textExp));
+    const int ymin = miniexp_to_int(miniexp_caddr(textExp));
+    const int xmax = miniexp_to_int(miniexp_cadddr(textExp));
+    const int ymax = miniexp_to_int(miniexp_caddddr(textExp));
+
+    if(rect.intersects(QRect(xmin, size.height() - ymax, xmax - xmin, ymax - ymin)))
+    {
+        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_car(textExp)));
+
+        if(type == QLatin1String("word"))
         {
-            if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "word", 4) == 0)
-            {
-                return QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
-            }
-            else
-            {
-                QStringList text;
+            return QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
+        }
+        else
+        {
+            QStringList text;
 
-                for(int textN = 5; textN < textLength; ++textN)
-                {
-                    text.append(loadText(miniexp_nth(textN, textExp), rect, pageHeight));
-                }
+            textExp = skip(textExp, 5);
 
-                if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "line", 4) == 0)
-                {
-                    return text.join(" ");
-                }
-                else
-                {
-                    return text.join("\n");
-                }
+            for(miniexp_t textItem = miniexp_nil; miniexp_consp(textExp); textExp = miniexp_cdr(textExp))
+            {
+                textItem = miniexp_car(textExp);
+
+                text.append(loadText(textItem, size, rect));
             }
+
+            return type == QLatin1String("line") ? text.join(" ") : text.join("\n");
         }
     }
 
     return QString();
 }
 
-void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent, const QHash< QString, int >& indexByName)
+QList< QRectF > findText(miniexp_t pageTextExp, const QSizeF& size, const QTransform& transform, const QStringList& words, bool matchCase, bool wholeWords)
 {
-    const int outlineLength = miniexp_length(outlineExp);
-
-    for(int outlineN = qMax(0, offset); outlineN < outlineLength; ++outlineN)
+    if(words.isEmpty())
     {
-        miniexp_t bookmarkExp = miniexp_nth(outlineN, outlineExp);
-        const int bookmarkLength = miniexp_length(bookmarkExp);
+        return QList< QRectF >();
+    }
 
-        if(bookmarkLength <= 1 || !miniexp_stringp(miniexp_nth(0, bookmarkExp)) || !miniexp_stringp(miniexp_nth(1, bookmarkExp)))
+    const Qt::CaseSensitivity caseSensitivity = matchCase ? Qt::CaseSensitive : Qt::CaseInsensitive;
+
+    QRectF result;
+    int wordIndex = 0;
+
+    QList< miniexp_t > texts;
+    QList< QRectF > results;
+
+    texts.append(pageTextExp);
+
+    while(!texts.isEmpty())
+    {
+        miniexp_t textExp = texts.takeFirst();
+
+        if(miniexp_length(textExp) < 6 || !miniexp_symbolp(miniexp_car(textExp)))
         {
             continue;
         }
 
-        const QString title = QString::fromUtf8(miniexp_to_str(miniexp_nth(0, bookmarkExp)));
-        QString destination = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, bookmarkExp)));
+        const QString type = QString::fromUtf8(miniexp_to_name(miniexp_car(textExp)));
+
+        if(type == QLatin1String("word"))
+        {
+            const QString text = QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
+
+            int index = 0;
+
+            while((index = text.indexOf(words.at(wordIndex), index, caseSensitivity)) != -1)
+            {
+                const int nextIndex = index + words.at(wordIndex).length();
+
+                const bool wordBegins = index == 0 || !text.at(index - 1).isLetterOrNumber();
+                const bool wordEnds = nextIndex == text.length() || !text.at(nextIndex).isLetterOrNumber();
+
+                if(!wholeWords || (wordBegins && wordEnds))
+                {
+                    const int xmin = miniexp_to_int(miniexp_cadr(textExp));
+                    const int ymin = miniexp_to_int(miniexp_caddr(textExp));
+                    const int xmax = miniexp_to_int(miniexp_cadddr(textExp));
+                    const int ymax = miniexp_to_int(miniexp_caddddr(textExp));
+
+                    result = result.united(QRectF(xmin, size.height() - ymax, xmax - xmin, ymax - ymin));
+
+                    // Advance after partial match
+                    if(++wordIndex == words.size())
+                    {
+                        results.append(transform.mapRect(result));
+
+                        // Reset after full match
+                        result = QRectF();
+                        wordIndex = 0;
+                    }
+                }
+                else
+                {
+                    // Reset after malformed match
+                    result = QRectF();
+                    wordIndex = 0;
+                }
+
+                if((index = nextIndex) >= text.length())
+                {
+                    break;
+                }
+            }
+
+            if(index < 0)
+            {
+                // Reset after empty match
+                result = QRectF();
+                wordIndex = 0;
+            }
+        }
+        else
+        {
+            textExp = skip(textExp, 5);
+
+            for(miniexp_t textItem = miniexp_nil; miniexp_consp(textExp); textExp = miniexp_cdr(textExp))
+            {
+                textItem = miniexp_car(textExp);
+
+                texts.append(textItem);
+            }
+        }
+    }
+
+    return results;
+}
+
+void loadOutline(miniexp_t outlineExp, QStandardItem* parent, const QHash< QString, int >& indexByName)
+{
+    for(miniexp_t outlineItem = miniexp_nil; miniexp_consp(outlineExp); outlineExp = miniexp_cdr(outlineExp))
+    {
+        outlineItem = miniexp_car(outlineExp);
+
+        if(miniexp_length(outlineItem) < 2 || !miniexp_stringp(miniexp_car(outlineItem)) || !miniexp_stringp(miniexp_cadr(outlineItem)))
+        {
+            continue;
+        }
+
+        const QString title = QString::fromUtf8(miniexp_to_str(miniexp_car(outlineItem)));
+        QString destination = QString::fromUtf8(miniexp_to_str(miniexp_cadr(outlineItem)));
 
         if(!title.isEmpty() && !destination.isEmpty())
         {
@@ -295,7 +429,7 @@ void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent, const
                 QStandardItem* item = new QStandardItem(title);
                 item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 
-                item->setData(destinationPage, Qt::UserRole + 1);
+                item->setData(destinationPage, Document::PageRole);
 
                 QStandardItem* pageItem = item->clone();
                 pageItem->setText(QString::number(destinationPage));
@@ -303,15 +437,48 @@ void loadOutline(miniexp_t outlineExp, int offset, QStandardItem* parent, const
 
                 parent->appendRow(QList< QStandardItem* >() << item << pageItem);
 
-                if(bookmarkLength >= 3)
+                if(miniexp_length(outlineItem) > 2)
                 {
-                    loadOutline(bookmarkExp, 2, item, indexByName);
+                    loadOutline(skip(outlineItem, 2), item, indexByName);
                 }
             }
         }
     }
 }
 
+void loadProperties(miniexp_t annoExp, QStandardItemModel* model)
+{
+    for(miniexp_t annoItem = miniexp_nil; miniexp_consp(annoExp); annoExp = miniexp_cdr(annoExp))
+    {
+        annoItem = miniexp_car(annoExp);
+
+        if(miniexp_length(annoItem) < 2 || qstrcmp(miniexp_to_name(miniexp_car(annoItem)), "metadata") != 0)
+        {
+            continue;
+        }
+
+        annoItem = miniexp_cdr(annoItem);
+
+        for(miniexp_t keyValueItem = miniexp_nil; miniexp_consp(annoItem); annoItem = miniexp_cdr(annoItem))
+        {
+            keyValueItem = miniexp_car(annoItem);
+
+            if(miniexp_length(keyValueItem) != 2)
+            {
+                continue;
+            }
+
+            const QString key = QString::fromUtf8(miniexp_to_name(miniexp_car(keyValueItem)));
+            const QString value = QString::fromUtf8(miniexp_to_str(miniexp_cadr(keyValueItem)));
+
+            if(!key.isEmpty() && !value.isEmpty())
+            {
+                model->appendRow(QList< QStandardItem* >() << new QStandardItem(key) << new QStandardItem(value));
+            }
+        }
+    }
+}
+
 } // anonymous
 
 namespace qpdfview
@@ -339,9 +506,8 @@ QSizeF DjVuPage::size() const
 
 QImage DjVuPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
 {
-    QMutexLocker mutexLocker(&m_parent->m_mutex);
+    LOCK_PAGE
 
-    ddjvu_status_t status;
     ddjvu_page_t* page = ddjvu_page_create_by_pageno(m_parent->m_document, m_index);
 
     if(page == 0)
@@ -349,6 +515,8 @@ QImage DjVuPage::render(qreal horizontalResolution, qreal verticalResolution, Ro
         return QImage();
     }
 
+    ddjvu_status_t status;
+
     while(true)
     {
         status = ddjvu_page_decoding_status(page);
@@ -440,12 +608,12 @@ QImage DjVuPage::render(qreal horizontalResolution, qreal verticalResolution, Ro
 
 QList< Link* > DjVuPage::links() const
 {
-    QMutexLocker mutexLocker(&m_parent->m_mutex);
+    LOCK_PAGE
 
-    miniexp_t pageAnnoExp;
+    miniexp_t pageAnnoExp = miniexp_nil;
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         while(true)
         {
@@ -462,10 +630,10 @@ QList< Link* > DjVuPage::links() const
         }
     }
 
-    QList< Link* > links = loadLinks(pageAnnoExp, m_size, m_index, m_parent->m_indexByName);
+    const QList< Link* > links = loadLinks(pageAnnoExp, m_size, m_index, m_parent->m_indexByName);
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         ddjvu_miniexp_release(m_parent->m_document, pageAnnoExp);
     }
@@ -475,12 +643,12 @@ QList< Link* > DjVuPage::links() const
 
 QString DjVuPage::text(const QRectF& rect) const
 {
-    QMutexLocker mutexLocker(&m_parent->m_mutex);
+    LOCK_PAGE
 
-    miniexp_t pageTextExp;
+    miniexp_t pageTextExp = miniexp_nil;
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         while(true)
         {
@@ -497,25 +665,27 @@ QString DjVuPage::text(const QRectF& rect) const
         }
     }
 
-    const QString text = loadText(pageTextExp, QTransform::fromScale(m_resolution / 72.0, m_resolution / 72.0).mapRect(rect).toRect(), m_size.height());
+    const QTransform transform = QTransform::fromScale(m_resolution / 72.0, m_resolution / 72.0);
+
+    const QString text = loadText(pageTextExp, m_size, transform.mapRect(rect)).simplified();
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
     }
 
-    return text.trimmed();
+    return text.simplified();
 }
 
-QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
+QList< QRectF > DjVuPage::search(const QString& text, bool matchCase, bool wholeWords) const
 {
-    QMutexLocker mutexLocker(&m_parent->m_mutex);
+    LOCK_PAGE
 
-    miniexp_t pageTextExp;
+    miniexp_t pageTextExp = miniexp_nil;
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         while(true)
         {
@@ -532,66 +702,13 @@ QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
         }
     }
 
-    QList< miniexp_t > words;
-    QList< QRectF > results;
-
-    words.append(pageTextExp);
-
-    QRectF rect;
-    int index = 0;
-
     const QTransform transform = QTransform::fromScale(72.0 / m_resolution, 72.0 / m_resolution);
+    const QStringList words = text.split(QRegExp(QLatin1String("\\W+")), QString::SkipEmptyParts);
 
-    while(!words.isEmpty())
-    {
-        miniexp_t textExp = words.takeFirst();
-
-        const int textLength = miniexp_length(textExp);
-
-        if(textLength >= 6 && miniexp_symbolp(miniexp_nth(0, textExp)))
-        {
-            if(qstrncmp(miniexp_to_name(miniexp_nth(0, textExp)), "word", 4) == 0)
-            {
-                const QString word = QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
-
-                index = word.indexOf(text, index, matchCase ? Qt::CaseSensitive : Qt::CaseInsensitive);
-
-                if(index != -1)
-                {
-                    const int xmin = miniexp_to_int(miniexp_nth(1, textExp));
-                    const int ymin = miniexp_to_int(miniexp_nth(2, textExp));
-                    const int xmax = miniexp_to_int(miniexp_nth(3, textExp));
-                    const int ymax = miniexp_to_int(miniexp_nth(4, textExp));
-
-                    index += text.length();
-                    rect = rect.united(QRectF(xmin, m_size.height() - ymax, xmax - xmin, ymax - ymin));
-
-                    if(index == word.length() || !word.at(index).isLetter())
-                    {
-                        results.append(transform.mapRect(rect));
-
-                        rect = QRectF();
-                        index = 0;
-                    }
-                }
-                else
-                {
-                    rect = QRectF();
-                    index = 0;
-                }
-            }
-            else
-            {
-                for(int textN = 5; textN < textLength; ++textN)
-                {
-                    words.append(miniexp_nth(textN, textExp));
-                }
-            }
-        }
-    }
+    const QList< QRectF > results = findText(pageTextExp, m_size, transform, words, matchCase, wholeWords);
 
     {
-        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+        LOCK_PAGE_GLOBAL
 
         ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
     }
@@ -613,19 +730,7 @@ DjVuDocument::DjVuDocument(QMutex* globalMutex, ddjvu_context_t* context, ddjvu_
     ddjvu_format_set_row_order(m_format, 1);
     ddjvu_format_set_y_direction(m_format, 1);
 
-    const int fileNum = ddjvu_document_get_filenum(m_document);
-
-    for(int index = 0; index < fileNum; ++index)
-    {
-        ddjvu_fileinfo_t fileinfo;
-
-        if(ddjvu_document_get_fileinfo(m_document, index, &fileinfo) != DDJVU_JOB_OK || fileinfo.type != 'P')
-        {
-            continue;
-        }
-
-        m_indexByName[QString::fromUtf8(fileinfo.id)] = m_indexByName[QString::fromUtf8(fileinfo.name)] = m_indexByName[QString::fromUtf8(fileinfo.title)] = fileinfo.pageno;
-    }
+    prepareIndexByName();
 }
 
 DjVuDocument::~DjVuDocument()
@@ -637,14 +742,14 @@ DjVuDocument::~DjVuDocument()
 
 int DjVuDocument::numberOfPages() const
 {
-    QMutexLocker mutexLocker(&m_mutex);
+    LOCK_DOCUMENT
 
     return ddjvu_document_get_pagenum(m_document);
 }
 
 Page* DjVuDocument::page(int index) const
 {
-    QMutexLocker mutexLocker(&m_mutex);
+    LOCK_DOCUMENT
 
     ddjvu_status_t status;
     ddjvu_pageinfo_t pageinfo;
@@ -673,7 +778,7 @@ Page* DjVuDocument::page(int index) const
 
 QStringList DjVuDocument::saveFilter() const
 {
-    return QStringList() << "DjVu (*.djvu *.djv)";
+    return QStringList() << QLatin1String("DjVu (*.djvu *.djv)");
 }
 
 bool DjVuDocument::canSave() const
@@ -685,10 +790,18 @@ bool DjVuDocument::save(const QString& filePath, bool withChanges) const
 {
     Q_UNUSED(withChanges);
 
-    QMutexLocker mutexLocker(&m_mutex);
+    LOCK_DOCUMENT
+
+#ifdef _MSC_VER
+
+    FILE* file = _wfopen(reinterpret_cast< const wchar_t* >(filePath.utf16()), L"w+");
+
+#else
 
     FILE* file = fopen(QFile::encodeName(filePath), "w+");
 
+#endif // _MSC_VER
+
     if(file == 0)
     {
         return false;
@@ -710,12 +823,12 @@ void DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
 {
     Document::loadOutline(outlineModel);
 
-    QMutexLocker mutexLocker(&m_mutex);
+    LOCK_DOCUMENT
 
-    miniexp_t outlineExp;
+    miniexp_t outlineExp = miniexp_nil;
 
     {
-        QMutexLocker globalMutexLocker(m_globalMutex);
+        LOCK_DOCUMENT_GLOBAL
 
         while(true)
         {
@@ -732,13 +845,15 @@ void DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
         }
     }
 
-    if(miniexp_length(outlineExp) > 1 && qstrncmp(miniexp_to_name(miniexp_nth(0, outlineExp)), "bookmarks", 9) == 0)
+    if(miniexp_length(outlineExp) < 2 || qstrcmp(miniexp_to_name(miniexp_car(outlineExp)), "bookmarks") != 0)
     {
-        ::loadOutline(outlineExp, 1, outlineModel->invisibleRootItem(), m_indexByName);
+        return;
     }
 
+    ::loadOutline(skip(outlineExp, 1), outlineModel->invisibleRootItem(), m_indexByName);
+
     {
-        QMutexLocker globalMutexLocker(m_globalMutex);
+        LOCK_DOCUMENT_GLOBAL
 
         ddjvu_miniexp_release(m_document, outlineExp);
     }
@@ -748,14 +863,12 @@ void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
 {
     Document::loadProperties(propertiesModel);
 
-    QMutexLocker mutexLocker(&m_mutex);
+    LOCK_DOCUMENT
 
-    propertiesModel->setColumnCount(2);
-
-    miniexp_t annoExp;
+    miniexp_t annoExp = miniexp_nil;
 
     {
-        QMutexLocker globalMutexLocker(m_globalMutex);
+        LOCK_DOCUMENT_GLOBAL
 
         while(true)
         {
@@ -772,41 +885,27 @@ void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
         }
     }
 
-    const int annoLength = miniexp_length(annoExp);
+    ::loadProperties(annoExp, propertiesModel);
 
-    for(int annoN = 0; annoN < annoLength; ++annoN)
     {
-        miniexp_t listExp = miniexp_nth(annoN, annoExp);
-        const int listLength = miniexp_length(listExp);
+        LOCK_DOCUMENT_GLOBAL
 
-        if(listLength <= 1 || qstrncmp(miniexp_to_name(miniexp_nth(0, listExp)), "metadata", 8) != 0)
+        ddjvu_miniexp_release(m_document, annoExp);
+    }
+}
+
+void DjVuDocument::prepareIndexByName()
+{
+    for(int index = 0, count = ddjvu_document_get_filenum(m_document); index < count; ++index)
+    {
+        ddjvu_fileinfo_t fileinfo;
+
+        if(ddjvu_document_get_fileinfo(m_document, index, &fileinfo) != DDJVU_JOB_OK || fileinfo.type != 'P')
         {
             continue;
         }
 
-        for(int listN = 1; listN < listLength; ++listN)
-        {
-            miniexp_t keyValueExp = miniexp_nth(listN, listExp);
-
-            if(miniexp_length(keyValueExp) != 2)
-            {
-                continue;
-            }
-
-            const QString key = QString::fromUtf8(miniexp_to_name(miniexp_nth(0, keyValueExp)));
-            const QString value = QString::fromUtf8(miniexp_to_str(miniexp_nth(1, keyValueExp)));
-
-            if(!key.isEmpty() && !value.isEmpty())
-            {
-                propertiesModel->appendRow(QList< QStandardItem* >() << new QStandardItem(key) << new QStandardItem(value));
-            }
-        }
-    }
-
-    {
-        QMutexLocker globalMutexLocker(m_globalMutex);
-
-        ddjvu_miniexp_release(m_document, annoExp);
+        m_indexByName[QString::fromUtf8(fileinfo.id)] = m_indexByName[QString::fromUtf8(fileinfo.name)] = m_indexByName[QString::fromUtf8(fileinfo.title)] = fileinfo.pageno;
     }
 }
 
