@@ -56,7 +56,8 @@ namespace Model
         QList< Link* > links() const;
 
         QString text(const QRectF& rect) const;
-        QList< QRectF > search(const QString& text, bool matchCase) const;
+
+        QList< QRectF > search(const QString& text, bool matchCase, bool wholeWords) const;
 
     private:
         Q_DISABLE_COPY(DjVuPage)
@@ -105,6 +106,8 @@ namespace Model
 
         QHash< QString, int > m_indexByName;
 
+        void prepareIndexByName();
+
     };
 }
 
