@@ -2,7 +2,7 @@
 
 Copyright 2014 S. Razi Alavizadeh
 Copyright 2013 Thomas Etter
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2015 Adam Reichold
 Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
@@ -63,6 +63,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "pageitem.h"
 #include "thumbnailitem.h"
 #include "presentationview.h"
+#include "searchmodel.h"
 #include "searchtask.h"
 #include "miscellaneous.h"
 #include "documentlayout.h"
@@ -117,20 +118,20 @@ QString intToRoman(int number)
 
     static const romandata_t romandata[] =
     {
-        1000, "m",
-        900, "cm",
-        500, "d",
-        400, "cd",
-        100, "c",
-        90, "xc",
-        50, "l",
-        40, "xl",
-        10, "x",
-        9, "ix",
-        5, "v",
-        4, "iv",
-        1, "i",
-        0, NULL
+        { 1000, "m" },
+        { 900, "cm" },
+        { 500, "d" },
+        { 400, "cd" },
+        { 100, "c" },
+        { 90, "xc" },
+        { 50, "l" },
+        { 40, "xl" },
+        { 10, "x" },
+        { 9, "ix" },
+        { 5, "v" },
+        { 4, "iv" },
+        { 1, "i" },
+        { 0, NULL }
     };
 
     if(number >= 4000)
@@ -213,6 +214,46 @@ bool modifiersUseMouseButton(Settings* settings, Qt::MouseButton mouseButton)
     return ((settings->documentView().zoomModifiers() | settings->documentView().rotateModifiers() | settings->documentView().scrollModifiers()) & mouseButton) != 0;
 }
 
+inline int pageOfResult(const QModelIndex& index)
+{
+    return index.data(SearchModel::PageRole).toInt();
+}
+
+inline QRectF rectOfResult(const QModelIndex& index)
+{
+    return index.data(SearchModel::RectRole).toRectF();
+}
+
+void saveExpandedPaths(const QAbstractItemModel* model, QSet< QString >& paths, const QModelIndex& index, QString path)
+{
+    path += index.data(Qt::DisplayRole).toString();
+
+    if(model->data(index, Model::Document::ExpansionRole).toBool())
+    {
+        paths.insert(path);
+    }
+
+    for(int row = 0, rowCount = model->rowCount(index); row < rowCount; ++row)
+    {
+        saveExpandedPaths(model, paths, model->index(row, 0, index), path);
+    }
+}
+
+void restoreExpandedPaths(QAbstractItemModel* model, const QSet< QString >& paths, const QModelIndex& index, QString path)
+{
+    path += index.data(Qt::DisplayRole).toString();
+
+    if(paths.contains(path))
+    {
+        model->setData(index, true, Model::Document::ExpansionRole);
+    }
+
+    for(int row = 0, rowCount = model->rowCount(index); row < rowCount; ++row)
+    {
+        restoreExpandedPaths(model, paths, model->index(row, 0, index), path);
+    }
+}
+
 } // anonymous
 
 namespace qpdfview
@@ -220,6 +261,7 @@ namespace qpdfview
 
 Settings* DocumentView::s_settings = 0;
 ShortcutHandler* DocumentView::s_shortcutHandler = 0;
+SearchModel* DocumentView::s_searchModel = 0;
 
 DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_autoRefreshWatcher(0),
@@ -239,6 +281,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_scaleFactor(1.0),
     m_rotation(RotateBy0),
     m_invertColors(false),
+    m_convertToGrayscale(false),
     m_highlightAll(false),
     m_rubberBandMode(ModifiersMode),
     m_pageItems(),
@@ -248,8 +291,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_thumbnailsScene(0),
     m_outlineModel(0),
     m_propertiesModel(0),
-    m_results(),
-    m_currentResult(m_results.end()),
+    m_currentResult(),
     m_searchTask(0)
 {
     if(s_settings == 0)
@@ -262,12 +304,17 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
         s_shortcutHandler = ShortcutHandler::instance();
     }
 
+    if(s_searchModel == 0)
+    {
+        s_searchModel = SearchModel::instance();
+    }
+
     setScene(new QGraphicsScene(this));
 
     setAcceptDrops(false);
     setDragMode(QGraphicsView::ScrollHandDrag);
 
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(on_verticalScrollBar_valueChanged()));
+    connectVerticalScrollBar();
 
     m_thumbnailsScene = new QGraphicsScene(this);
 
@@ -287,8 +334,8 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_searchTask = new SearchTask(this);
 
     connect(m_searchTask, SIGNAL(finished()), SIGNAL(searchFinished()));
-    connect(m_searchTask, SIGNAL(progressChanged(int)), SIGNAL(searchProgressChanged(int)));
 
+    connect(m_searchTask, SIGNAL(progressChanged(int)), SLOT(on_searchTask_progressChanged(int)));
     connect(m_searchTask, SIGNAL(resultsReady(int,QList<QRectF>)), SLOT(on_searchTask_resultsReady(int,QList<QRectF>)));
 
     // auto-refresh
@@ -329,6 +376,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_rotation = s_settings->documentView().rotation();
 
     m_invertColors = s_settings->documentView().invertColors();
+    m_convertToGrayscale = s_settings->documentView().convertToGrayscale();
     m_highlightAll = s_settings->documentView().highlightAll();
 }
 
@@ -337,6 +385,8 @@ DocumentView::~DocumentView()
     m_searchTask->cancel();
     m_searchTask->wait();
 
+    s_searchModel->clearResults(this);
+
     qDeleteAll(m_pageItems);
     qDeleteAll(m_thumbnailItems);
 
@@ -350,14 +400,14 @@ void DocumentView::setFirstPage(int firstPage)
     {
         m_firstPage = firstPage;
 
-        qDeleteAll(m_thumbnailItems);
+        for(int index = 0; index < m_thumbnailItems.count(); ++index)
+        {
+            m_thumbnailItems.at(index)->setText(pageLabelFromNumber(index + 1));
+        }
 
-        prepareThumbnails();
         prepareThumbnailsScene();
 
-        m_document->loadOutline(m_outlineModel);
-
-        if(m_outlineModel->rowCount() == 0)
+        if(m_outlineModel->invisibleRootItem()->data().toBool())
         {
             loadFallbackOutline();
         }
@@ -381,7 +431,7 @@ QString DocumentView::pageLabelFromNumber(int number) const
 
     if(hasFrontMatter())
     {
-        if (number < m_firstPage)
+        if(number < m_firstPage)
         {
             label = number < 4000 ? intToRoman(number) : defaultPageLabelFromNumber(-number);
         }
@@ -445,6 +495,33 @@ int DocumentView::pageNumberFromLabel(const QString& label) const
     return locale().toInt(label);
 }
 
+QString DocumentView::title() const
+{
+    QString title;
+
+    if(s_settings->mainWindow().documentTitleAsTabTitle())
+    {
+        for(int row = 0, rowCount = m_propertiesModel->rowCount(); row < rowCount; ++row)
+        {
+            const QStandardItem* keyItem = m_propertiesModel->item(row, 0);
+            const QStandardItem* valueItem = m_propertiesModel->item(row, 1);
+
+            if(keyItem != 0 && valueItem != 0 && QLatin1String("Title") == keyItem->text())
+            {
+                title = valueItem->text();
+                break;
+            }
+        }
+    }
+
+    if(title.isEmpty())
+    {
+        title = m_fileInfo.completeBaseName();
+    }
+
+    return title;
+}
+
 QStringList DocumentView::openFilter()
 {
     QStringList openFilter;
@@ -618,7 +695,7 @@ void DocumentView::setInvertColors(bool invertColors)
             page->setInvertColors(m_invertColors);
         }
 
-        foreach(PageItem* page, m_thumbnailItems)
+        foreach(ThumbnailItem* page, m_thumbnailItems)
         {
             page->setInvertColors(m_invertColors);
         }
@@ -631,20 +708,51 @@ void DocumentView::setInvertColors(bool invertColors)
     }
 }
 
+void DocumentView::setConvertToGrayscale(bool convertToGrayscale)
+{
+    if(m_convertToGrayscale != convertToGrayscale)
+    {
+        m_convertToGrayscale = convertToGrayscale;
+
+        foreach(PageItem* page, m_pageItems)
+        {
+            page->setConvertToGrayscale(m_convertToGrayscale);
+        }
+
+        foreach(ThumbnailItem* page, m_thumbnailItems)
+        {
+            page->setConvertToGrayscale(m_convertToGrayscale);
+        }
+
+        emit convertToGrayscaleChanged(m_convertToGrayscale);
+
+        s_settings->documentView().setConvertToGrayscale(m_convertToGrayscale);
+    }
+}
+
 void DocumentView::setHighlightAll(bool highlightAll)
 {
     if(m_highlightAll != highlightAll)
     {
         m_highlightAll = highlightAll;
 
-        for(int index = 0; index < m_pageItems.count(); ++index)
+        if(m_highlightAll)
         {
-            m_pageItems.at(index)->setHighlights(m_highlightAll ? m_results.values(index) : QList< QRectF >());
-        }
+            for(int index = 0; index < m_pages.count(); ++index)
+            {
+                const QList< QRectF >& results = s_searchModel->resultsOnPage(this, index + 1);
 
-        for(int index = 0; index < m_thumbnailItems.count(); ++index)
+                m_pageItems.at(index)->setHighlights(results);
+                m_thumbnailItems.at(index)->setHighlights(results);
+            }
+        }
+        else
         {
-            m_thumbnailItems.at(index)->setHighlights(m_highlightAll ? m_results.values(index) : QList< QRectF >());
+            for(int index = 0; index < m_pages.count(); ++index)
+            {
+                m_pageItems.at(index)->setHighlights(QList< QRectF >());
+                m_thumbnailItems.at(index)->setHighlights(QList< QRectF >());
+            }
         }
 
         emit highlightAllChanged(m_highlightAll);
@@ -697,6 +805,41 @@ QStandardItemModel* DocumentView::fontsModel() const
     return fontsModel;
 }
 
+QString DocumentView::searchText() const
+{
+    return m_searchTask->text();
+}
+
+bool DocumentView::searchMatchCase() const
+{
+    return m_searchTask->matchCase();
+}
+
+bool DocumentView::searchWholeWords() const
+{
+    return m_searchTask->wholeWords();
+}
+
+QPair< QString, QString > DocumentView::searchContext(int page, const QRectF& rect) const
+{
+    if(page < 1 || page > m_pages.size() || rect.isEmpty())
+    {
+        return qMakePair(QString(), QString());
+    }
+
+    // Fetch at most half of a line as centered on the given rectangle as possible.
+    const qreal pageWidth = m_pages.at(page - 1)->size().width();
+    const qreal width = qMax(rect.width(), pageWidth / qreal(2));
+    const qreal x = qBound(qreal(0), rect.x() + rect.width() / qreal(2) - width / qreal(2), pageWidth - width);
+
+    const QRectF surroundingRect(x, rect.top(), width, rect.height());
+
+    const QString& matchedText = m_pages.at(page - 1)->cachedText(rect);
+    const QString& surroundingText = m_pages.at(page - 1)->cachedText(surroundingRect);
+
+    return qMakePair(matchedText, surroundingText);
+}
+
 void DocumentView::show()
 {
     QGraphicsView::show();
@@ -710,9 +853,13 @@ bool DocumentView::open(const QString& filePath)
 
     if(document != 0)
     {
-        if(!checkDocument(filePath, document))
+        QVector< Model::Page* > pages;
+
+        if(!checkDocument(filePath, document, pages))
         {
             delete document;
+            qDeleteAll(pages);
+
             return false;
         }
 
@@ -724,7 +871,7 @@ bool DocumentView::open(const QString& filePath)
         m_past.clear();
         m_future.clear();
 
-        prepareDocument(document);
+        prepareDocument(document, pages);
 
         loadDocumentDefaults();
 
@@ -756,9 +903,13 @@ bool DocumentView::refresh()
 
     if(document != 0)
     {
-        if(!checkDocument(m_fileInfo.filePath(), document))
+        QVector< Model::Page* > pages;
+
+        if(!checkDocument(m_fileInfo.filePath(), document, pages))
         {
             delete document;
+            qDeleteAll(pages);
+
             return false;
         }
 
@@ -769,7 +920,12 @@ bool DocumentView::refresh()
 
         m_currentPage = qMin(m_currentPage, document->numberOfPages());
 
-        prepareDocument(document);
+        QSet< QString > expandedPaths;
+        saveExpandedPaths(m_outlineModel, expandedPaths, QModelIndex(), QString());
+
+        prepareDocument(document, pages);
+
+        restoreExpandedPaths(m_outlineModel, expandedPaths, QModelIndex(), QString());
 
         prepareScene();
         prepareView(left, top);
@@ -946,103 +1102,72 @@ void DocumentView::temporaryHighlight(int page, const QRectF& highlight)
     }
 }
 
-void DocumentView::startSearch(const QString& text, bool matchCase)
+void DocumentView::startSearch(const QString& text, bool matchCase, bool wholeWords)
 {
     cancelSearch();
+    clearResults();
 
-    m_searchTask->start(m_pages, text, matchCase, m_currentPage);
+    m_searchTask->start(m_pages, text, matchCase, wholeWords, m_currentPage);
 }
 
 void DocumentView::cancelSearch()
 {
     m_searchTask->cancel();
     m_searchTask->wait();
+}
 
-    m_results.clear();
-    m_currentResult = m_results.end();
+void DocumentView::clearResults()
+{
+    s_searchModel->clearResults(this);
+
+    m_currentResult = QModelIndex();
+
+    m_highlight->setVisible(false);
 
     foreach(PageItem* page, m_pageItems)
     {
         page->setHighlights(QList< QRectF >());
     }
 
-    foreach(PageItem* page, m_thumbnailItems)
+    foreach(ThumbnailItem* page, m_thumbnailItems)
     {
         page->setHighlights(QList< QRectF >());
     }
 
-    prepareThumbnailsScene();
-
-    m_highlight->setVisible(false);
+    if(s_settings->documentView().limitThumbnailsToResults())
+    {
+        prepareThumbnailsScene();
+    }
 }
 
 void DocumentView::findPrevious()
 {
-    if(m_currentResult != m_results.end())
-    {
-        if(m_layout->leftIndex(m_currentResult.key()) == m_currentPage - 1)
-        {
-            m_currentResult = previousResult(m_currentResult);
-        }
-        else
-        {
-            m_currentResult = previousResult(m_results.upperBound(m_currentPage - 1));
-        }
-    }
-    else
-    {
-        m_currentResult = previousResult(m_results.upperBound(m_currentPage - 1));
-    }
+    checkResult();
 
-    if(m_currentResult == m_results.end())
-    {
-        m_currentResult = previousResult(m_results.end());
-    }
+    m_currentResult = s_searchModel->findResult(this, m_currentResult, m_currentPage, SearchModel::FindPrevious);
 
-    if(m_currentResult != m_results.end())
-    {
-        jumpToPage(m_currentResult.key() + 1);
-
-        prepareHighlight(m_currentResult.key(), m_currentResult.value());
-    }
-    else
-    {
-        m_highlight->setVisible(false);
-    }
+    applyResult();
 }
 
 void DocumentView::findNext()
 {
-    if(m_currentResult != m_results.end())
-    {
-        if(m_layout->leftIndex(m_currentResult.key()) == m_currentPage - 1)
-        {
-            ++m_currentResult;
-        }
-        else
-        {
-            m_currentResult = m_results.lowerBound(m_currentPage - 1);
-        }
-    }
-    else
-    {
-        m_currentResult = m_results.lowerBound(m_currentPage - 1);
-    }
+    checkResult();
 
-    if(m_currentResult == m_results.end())
-    {
-        m_currentResult = m_results.begin();
-    }
+    m_currentResult = s_searchModel->findResult(this, m_currentResult, m_currentPage, SearchModel::FindNext);
 
-    if(m_currentResult != m_results.end())
-    {
-        jumpToPage(m_currentResult.key() + 1);
+    applyResult();
+}
 
-        prepareHighlight(m_currentResult.key(), m_currentResult.value());
-    }
-    else
+void DocumentView::findResult(const QModelIndex& index)
+{
+    const int page = pageOfResult(index);
+    const QRectF rect = rectOfResult(index);
+
+    if(page >= 1 && page <= m_pages.count() && !rect.isEmpty())
     {
-        m_highlight->setVisible(false);
+        m_currentResult = index;
+
+        applyResult();
     }
 }
 
@@ -1240,22 +1365,26 @@ void DocumentView::on_temporaryHighlight_timeout()
     m_highlight->setVisible(false);
 }
 
-void DocumentView::on_searchTask_resultsReady(int index, QList< QRectF > results)
+void DocumentView::on_searchTask_progressChanged(int progress)
+{
+    s_searchModel->updateProgress(this);
+
+    emit searchProgressChanged(progress);
+}
+
+void DocumentView::on_searchTask_resultsReady(int index, const QList< QRectF >& results)
 {
     if(m_searchTask->wasCanceled())
     {
         return;
     }
 
-    while(!results.isEmpty())
-    {
-        m_results.insertMulti(index, results.takeLast());
-    }
+    s_searchModel->insertResults(this, index + 1, results);
 
     if(m_highlightAll)
     {
-        m_pageItems.at(index)->setHighlights(m_results.values(index));
-        m_thumbnailItems.at(index)->setHighlights(m_results.values(index));
+        m_pageItems.at(index)->setHighlights(results);
+        m_thumbnailItems.at(index)->setHighlights(results);
     }
 
     if(s_settings->documentView().limitThumbnailsToResults())
@@ -1263,7 +1392,7 @@ void DocumentView::on_searchTask_resultsReady(int index, QList< QRectF > results
         prepareThumbnailsScene();
     }
 
-    if(m_results.contains(index) && m_currentResult == m_results.end())
+    if(!results.isEmpty() && !m_currentResult.isValid())
     {
         setFocus();
 
@@ -1306,6 +1435,13 @@ void DocumentView::on_pages_linkClicked(bool newTab, int page, qreal left, qreal
     }
 }
 
+void DocumentView::on_pages_linkClicked(bool newTab, const QString& fileName, int page)
+{
+    const QString filePath = QFileInfo(fileName).isAbsolute() ? fileName : m_fileInfo.dir().filePath(fileName);
+
+    emit linkClicked(newTab, filePath, page);
+}
+
 void DocumentView::on_pages_linkClicked(const QString& url)
 {
     if(s_settings->documentView().openUrl())
@@ -1325,13 +1461,6 @@ void DocumentView::on_pages_linkClicked(const QString& url)
     }
 }
 
-void DocumentView::on_pages_linkClicked(const QString& fileName, int page)
-{
-    const QString filePath = QFileInfo(fileName).isAbsolute() ? fileName : m_fileInfo.dir().filePath(fileName);
-
-    emit linkClicked(filePath, page);
-}
-
 void DocumentView::on_pages_rubberBandFinished()
 {
     setRubberBandMode(ModifiersMode);
@@ -1413,13 +1542,13 @@ void DocumentView::on_pages_wasModified()
 
 void DocumentView::resizeEvent(QResizeEvent* event)
 {
+    qreal left = 0.0, top = 0.0;
+    saveLeftAndTop(left, top);
+
     QGraphicsView::resizeEvent(event);
 
     if(m_scaleMode != ScaleFactorMode)
     {
-        qreal left = 0.0, top = 0.0;
-        saveLeftAndTop(left, top);
-
         prepareScene();
         prepareView(left, top);
     }
@@ -1427,6 +1556,15 @@ void DocumentView::resizeEvent(QResizeEvent* event)
 
 void DocumentView::keyPressEvent(QKeyEvent* event)
 {
+    foreach(const PageItem* page, m_pageItems)
+    {
+        if(page->showsAnnotationOverlay() || page->showsFormFieldOverlay())
+        {
+            QGraphicsView::keyPressEvent(event);
+            return;
+        }
+    }
+
     const QKeySequence keySequence(event->modifiers() + event->key());
 
     int maskedKey = -1;
@@ -1469,14 +1607,6 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
     }
     else
     {
-        foreach(const PageItem* page, m_pageItems)
-        {
-            if(page->showsAnnotationOverlay() || page->showsFormFieldOverlay())
-            {
-                return;
-            }
-        }
-
         if(!m_continuousMode)
         {
             if(maskedKey == Qt::Key_PageUp && verticalScrollBar()->value() == verticalScrollBar()->minimum() && m_currentPage != 1)
@@ -1521,6 +1651,24 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
     }
 }
 
+void DocumentView::mousePressEvent(QMouseEvent* event)
+{
+    if(event->button() == Qt::XButton1)
+    {
+        event->accept();
+
+        jumpBackward();
+    }
+    else if(event->button() == Qt::XButton2)
+    {
+        event->accept();
+
+        jumpForward();
+    }
+
+    QGraphicsView::mousePressEvent(event);
+}
+
 void DocumentView::wheelEvent(QWheelEvent* event)
 {
     const Qt::KeyboardModifiers zoomModifiers = s_settings->documentView().zoomModifiers();
@@ -1882,15 +2030,15 @@ bool DocumentView::printUsingQt(QPrinter* printer, const PrintOptions& printOpti
 void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
 {
     const PageItem* page = m_pageItems.at(m_currentPage - 1);
-
     const QRectF boundingRect = page->boundingRect().translated(page->pos());
+
     const QPointF topLeft = mapToScene(viewport()->rect().topLeft());
 
     left = (topLeft.x() - boundingRect.x()) / boundingRect.width();
     top = (topLeft.y() - boundingRect.y()) / boundingRect.height();
 }
 
-bool DocumentView::checkDocument(const QString& filePath, Model::Document* document)
+bool DocumentView::checkDocument(const QString& filePath, Model::Document* document, QVector< Model::Page* >& pages)
 {
     if(document->isLocked())
     {
@@ -1902,18 +2050,38 @@ bool DocumentView::checkDocument(const QString& filePath, Model::Document* docum
         }
     }
 
-    if(document->numberOfPages() == 0)
+    const int numberOfPages = document->numberOfPages();
+
+    if(numberOfPages == 0)
     {
         qWarning() << "No pages were found in document at" << filePath;
 
         return false;
     }
 
+    pages.reserve(numberOfPages);
+
+    for(int index = 0; index < numberOfPages; ++index)
+    {
+        Model::Page* page = document->page(index);
+
+        if(page == 0)
+        {
+            qWarning() << "No page" << index << "was found in document at" << filePath;
+
+            return false;
+        }
+
+        pages.append(page);
+    }
+
     return true;
 }
 
 void DocumentView::loadFallbackOutline()
 {
+    m_outlineModel->clear();
+
     for(int page = 1; page <= m_pages.count(); ++page)
     {
         QStandardItem* item = new QStandardItem(tr("Page %1").arg(pageLabelFromNumber(page)));
@@ -1927,6 +2095,8 @@ void DocumentView::loadFallbackOutline()
 
         m_outlineModel->appendRow(QList< QStandardItem* >() << item << pageItem);
     }
+
+    m_outlineModel->invisibleRootItem()->setData(true); // Flags this as a fallback outline.
 }
 
 void DocumentView::loadDocumentDefaults()
@@ -1975,30 +2145,32 @@ void DocumentView::adjustScrollBarPolicy()
     }
 }
 
-void DocumentView::prepareDocument(Model::Document* document)
+void DocumentView::connectVerticalScrollBar()
+{
+    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
+}
+
+void DocumentView::disconnectVerticalScrollBar()
+{
+    disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
+}
+
+void DocumentView::prepareDocument(Model::Document* document, const QVector< Model::Page* >& pages)
 {
     m_prefetchTimer->blockSignals(true);
     m_prefetchTimer->stop();
 
     cancelSearch();
+    clearResults();
 
     qDeleteAll(m_pageItems);
     qDeleteAll(m_thumbnailItems);
 
-    qDeleteAll(m_pages);
-
     delete m_document;
     m_document = document;
 
-    const int numberOfPages = m_document->numberOfPages();
-
-    m_pages.clear();
-    m_pages.reserve(numberOfPages);
-
-    for(int index = 0; index < numberOfPages; ++index)
-    {
-        m_pages.append(m_document->page(index));
-    }
+    qDeleteAll(m_pages);
+    m_pages = pages;
 
     if(!m_autoRefreshWatcher->files().isEmpty())
     {
@@ -2049,8 +2221,8 @@ void DocumentView::preparePages()
         connect(page, SIGNAL(cropRectChanged()), SLOT(on_pages_cropRectChanged()));
 
         connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
+        connect(page, SIGNAL(linkClicked(bool,QString,int)), SLOT(on_pages_linkClicked(bool,QString,int)));
         connect(page, SIGNAL(linkClicked(QString)), SLOT(on_pages_linkClicked(QString)));
-        connect(page, SIGNAL(linkClicked(QString,int)), SLOT(on_pages_linkClicked(QString,int)));
 
         connect(page, SIGNAL(rubberBandFinished()), SLOT(on_pages_rubberBandFinished()));
 
@@ -2154,8 +2326,6 @@ void DocumentView::prepareScene()
 
 void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePage)
 {
-    const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
-
     qreal left = scene()->sceneRect().left();
     qreal top = scene()->sceneRect().top();
     qreal width = scene()->sceneRect().width();
@@ -2166,6 +2336,9 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
 
     visiblePage = visiblePage == 0 ? m_currentPage : visiblePage;
 
+    const int highlightIsOnPage = m_currentResult.isValid() ? pageOfResult(m_currentResult) : 0;
+    const bool highlightCurrentThumbnail = s_settings->documentView().highlightCurrentThumbnail();
+
     for(int index = 0; index < m_pageItems.count(); ++index)
     {
         PageItem* page = m_pageItems.at(index);
@@ -2198,7 +2371,7 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
             verticalValue = qFloor(boundingRect.top() + changeTop * boundingRect.height());
         }
 
-        if(m_currentResult != m_results.end() && m_currentResult.key() == index)
+        if(index == highlightIsOnPage - 1)
         {
             m_highlight->setPos(page->pos());
             m_highlight->setTransform(page->transform());
@@ -2230,9 +2403,9 @@ void DocumentView::prepareThumbnailsScene()
 
     for(int index = 0; index < m_thumbnailItems.count(); ++index)
     {
-        PageItem* page = m_thumbnailItems.at(index);
+        ThumbnailItem* page = m_thumbnailItems.at(index);
 
-        if(limitThumbnailsToResults && !m_results.isEmpty() && !m_results.contains(index))
+        if(limitThumbnailsToResults && s_searchModel->hasResults(this) && !s_searchModel->hasResultsOnPage(this, index + 1))
         {
             page->setVisible(false);
 
@@ -2295,16 +2468,36 @@ void DocumentView::prepareHighlight(int index, const QRectF& rect)
 
     m_highlight->setVisible(true);
 
-    disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
+    disconnectVerticalScrollBar();
     centerOn(m_highlight);
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
+    connectVerticalScrollBar();
 
     viewport()->update();
 }
 
-DocumentView::Results::iterator DocumentView::previousResult(const Results::iterator& result)
+void DocumentView::checkResult()
 {
-    return result != m_results.begin() ? result - 1 : m_results.end();
+    if(m_currentResult.isValid() && m_layout->currentPage(pageOfResult(m_currentResult)) != m_currentPage)
+    {
+        m_currentResult = QModelIndex();
+    }
+}
+
+void DocumentView::applyResult()
+{
+    if(m_currentResult.isValid())
+    {
+        const int page = pageOfResult(m_currentResult);
+        const QRectF rect = rectOfResult(m_currentResult);
+
+        jumpToPage(page);
+
+        prepareHighlight(page - 1, rect);
+    }
+    else
+    {
+        m_highlight->setVisible(false);
+    }
 }
 
 } // qpdfview
