@@ -27,6 +27,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QFileInfo>
 #include <QGraphicsView>
 #include <QMap>
+#include <QPersistentModelIndex>
 
 class QDomNode;
 class QFileSystemWatcher;
@@ -49,6 +50,7 @@ class Document;
 class Settings;
 class PageItem;
 class ThumbnailItem;
+class SearchModel;
 class SearchTask;
 class PresentationView;
 class ShortcutHandler;
@@ -62,67 +64,78 @@ public:
     explicit DocumentView(QWidget* parent = 0);
     ~DocumentView();
 
-    inline const QFileInfo& fileInfo() const { return m_fileInfo; }
-    inline bool wasModified() const { return m_wasModified; }
+    const QFileInfo& fileInfo() const { return m_fileInfo; }
+    bool wasModified() const { return m_wasModified; }
 
-    inline int numberOfPages() const { return m_pages.count(); }
-    inline int currentPage() const { return m_currentPage; }
+    int numberOfPages() const { return m_pages.count(); }
+    int currentPage() const { return m_currentPage; }
 
-    inline bool hasFrontMatter() const { return m_firstPage > 1; }
+    bool hasFrontMatter() const { return m_firstPage > 1; }
 
-    inline int firstPage() const { return m_firstPage; }
+    int firstPage() const { return m_firstPage; }
     void setFirstPage(int firstPage);
 
     QString defaultPageLabelFromNumber(int number) const;
     QString pageLabelFromNumber(int number) const;
     int pageNumberFromLabel(const QString& label) const;
 
+    QString title() const;
+
     static QStringList openFilter();
     QStringList saveFilter() const;
 
     bool canSave() const;
 
-    inline bool continuousMode() const { return m_continuousMode; }
+    bool continuousMode() const { return m_continuousMode; }
     void setContinuousMode(bool continuousMode);
 
     LayoutMode layoutMode() const;
     void setLayoutMode(LayoutMode layoutMode);
 
-    inline bool rightToLeftMode() const { return m_rightToLeftMode; }
+    bool rightToLeftMode() const { return m_rightToLeftMode; }
     void setRightToLeftMode(bool rightToLeftMode);
 
-    inline ScaleMode scaleMode() const { return m_scaleMode; }
+    ScaleMode scaleMode() const { return m_scaleMode; }
     void setScaleMode(ScaleMode scaleMode);
 
-    inline qreal scaleFactor() const { return m_scaleFactor; }
+    qreal scaleFactor() const { return m_scaleFactor; }
     void setScaleFactor(qreal scaleFactor);
 
-    inline Rotation rotation() const { return m_rotation; }
+    Rotation rotation() const { return m_rotation; }
     void setRotation(Rotation rotation);
 
-    inline bool invertColors() const { return m_invertColors; }
+    bool invertColors() const { return m_invertColors; }
     void setInvertColors(bool invertColors);
 
-    inline bool highlightAll() const { return m_highlightAll; }
+    bool convertToGrayscale() const { return m_convertToGrayscale; }
+    void setConvertToGrayscale(bool convertToGrayscale);
+
+    bool highlightAll() const { return m_highlightAll; }
     void setHighlightAll(bool highlightAll);
 
-    inline RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
+    RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
     void setRubberBandMode(RubberBandMode rubberBandMode);
 
     bool searchWasCanceled() const;
     int searchProgress() const;
 
-    inline Qt::Orientation thumbnailsOrientation() const { return m_thumbnailsOrientation; }
+    Qt::Orientation thumbnailsOrientation() const { return m_thumbnailsOrientation; }
     void setThumbnailsOrientation(Qt::Orientation thumbnailsOrientation);
 
-    inline const QVector< ThumbnailItem* >& thumbnailItems() const { return m_thumbnailItems; }
-    inline QGraphicsScene* thumbnailsScene() const { return m_thumbnailsScene; }
+    const QVector< ThumbnailItem* >& thumbnailItems() const { return m_thumbnailItems; }
+    QGraphicsScene* thumbnailsScene() const { return m_thumbnailsScene; }
 
-    inline QStandardItemModel* outlineModel() const { return m_outlineModel; }
-    inline QStandardItemModel* propertiesModel() const { return m_propertiesModel; }
+    QStandardItemModel* outlineModel() const { return m_outlineModel; }
+    QStandardItemModel* propertiesModel() const { return m_propertiesModel; }
 
     QStandardItemModel* fontsModel() const;
 
+    QString searchText() const;
+    bool searchMatchCase() const;
+    bool searchWholeWords() const;
+
+    QPair< QString, QString > searchContext(int page, const QRectF& rect) const;
+
 signals:
     void documentChanged();
     void documentModified();
@@ -140,9 +153,10 @@ signals:
     void rotationChanged(Rotation rotation);
 
     void linkClicked(int page);
-    void linkClicked(const QString& filePath, int page);
+    void linkClicked(bool newTab, const QString& filePath, int page);
 
     void invertColorsChanged(bool invertColors);
+    void convertToGrayscaleChanged(bool convertToGrayscale);
     void highlightAllChanged(bool highlightAll);
     void rubberBandModeChanged(RubberBandMode rubberBandMode);
 
@@ -172,11 +186,14 @@ public slots:
 
     void temporaryHighlight(int page, const QRectF& highlight);
 
-    void startSearch(const QString& text, bool matchCase = true);
+    void startSearch(const QString& text, bool matchCase, bool wholeWords);
     void cancelSearch();
 
+    void clearResults();
+
     void findPrevious();
     void findNext();
+    void findResult(const QModelIndex& index);
 
     void zoomIn();
     void zoomOut();
@@ -195,14 +212,15 @@ protected slots:
 
     void on_temporaryHighlight_timeout();
 
-    void on_searchTask_resultsReady(int index, QList< QRectF > results);
+    void on_searchTask_progressChanged(int progress);
+    void on_searchTask_resultsReady(int index, const QList< QRectF >& results);
 
     void on_pages_cropRectChanged();
     void on_thumbnails_cropRectChanged();
 
     void on_pages_linkClicked(bool newTab, int page, qreal left, qreal top);
+    void on_pages_linkClicked(bool newTab, const QString& fileName, int page);
     void on_pages_linkClicked(const QString& url);
-    void on_pages_linkClicked(const QString& fileName, int page);
 
     void on_pages_rubberBandFinished();
 
@@ -215,6 +233,7 @@ protected:
     void resizeEvent(QResizeEvent* event);
 
     void keyPressEvent(QKeyEvent* event);
+    void mousePressEvent(QMouseEvent* event);
     void wheelEvent(QWheelEvent* event);
 
     void contextMenuEvent(QContextMenuEvent* event);
@@ -271,6 +290,7 @@ private:
     Rotation m_rotation;
 
     bool m_invertColors;
+    bool m_convertToGrayscale;
     bool m_highlightAll;
     RubberBandMode m_rubberBandMode;
 
@@ -285,14 +305,16 @@ private:
     QStandardItemModel* m_outlineModel;
     QStandardItemModel* m_propertiesModel;
 
-    bool checkDocument(const QString& filePath, Model::Document* document);
+    bool checkDocument(const QString& filePath, Model::Document* document, QVector< Model::Page* >& pages);
 
     void loadFallbackOutline();
     void loadDocumentDefaults();
 
     void adjustScrollBarPolicy();
+    void connectVerticalScrollBar();
+    void disconnectVerticalScrollBar();
 
-    void prepareDocument(Model::Document* document);
+    void prepareDocument(Model::Document* document, const QVector< Model::Page* >& pages);
     void preparePages();
     void prepareThumbnails();
     void prepareBackground();
@@ -306,15 +328,15 @@ private:
 
     // search
 
-    typedef QMultiMap< int, QRectF > Results;
+    static SearchModel* s_searchModel;
 
-    Results m_results;
-    Results::iterator m_currentResult;
-
-    Results::iterator previousResult(const Results::iterator& result);
+    QPersistentModelIndex m_currentResult;
 
     SearchTask* m_searchTask;
 
+    void checkResult();
+    void applyResult();
+
 };
 
 } // qpdfview
