@@ -1,7 +1,7 @@
 /*
 
 Copyright 2014 S. Razi Alavizadeh
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2015 Adam Reichold
 Copyright 2012 Michał Trybus
 Copyright 2012 Alexander Volkov
 
@@ -42,6 +42,7 @@ class QFileInfo;
 class QModelIndex;
 class QShortcut;
 class QTableView;
+class QTreeView;
 class QWidgetAction;
 
 #include "global.h"
@@ -56,6 +57,7 @@ class TreeView;
 class ComboBox;
 class MappingSpinBox;
 class SearchLineEdit;
+class SearchableMenu;
 class RecentlyUsedMenu;
 class RecentlyClosedMenu;
 class BookmarkModel;
@@ -67,7 +69,7 @@ class MainWindow : public QMainWindow
     Q_OBJECT
 
     friend class MainWindowAdaptor;
-    
+
 public:
     explicit MainWindow(QWidget* parent = 0);
 
@@ -75,6 +77,8 @@ public:
     QMenu* createPopupMenu();
 
 public slots:
+    void show();
+
     bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
     bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
 
@@ -88,6 +92,7 @@ protected slots:
     void on_tabWidget_tabContextMenuRequested(const QPoint& globalPos, int index);
 
     void on_currentTab_documentChanged();
+    void on_currentTab_documentModified();
 
     void on_currentTab_numberOfPagesChaned(int numberOfPages);
     void on_currentTab_currentPageChanged(int currentPage);
@@ -102,9 +107,10 @@ protected slots:
     void on_currentTab_rotationChanged(Rotation rotation);
 
     void on_currentTab_linkClicked(int page);
-    void on_currentTab_linkClicked(const QString& filePath, int page);
+    void on_currentTab_linkClicked(bool newTab, const QString& filePath, int page);
 
     void on_currentTab_invertColorsChanged(bool invertColors);
+    void on_currentTab_convertToGrayscale(bool convertToGrayscale);
     void on_currentTab_highlightAllChanged(bool highlightAll);
     void on_currentTab_rubberBandModeChanged(RubberBandMode rubberBandMode);
 
@@ -122,6 +128,7 @@ protected slots:
 
     void on_open_triggered();
     void on_openInNewTab_triggered();
+    void on_openCopyInNewTab_triggered();
     void on_openContainingFolder_triggered();
     void on_refresh_triggered();
     void on_saveCopy_triggered();
@@ -170,6 +177,7 @@ protected slots:
     void on_rotateRight_triggered();
 
     void on_invertColors_triggered(bool checked);
+    void on_convertToGrayscale_triggered(bool checked);
 
     void on_fonts_triggered();
 
@@ -203,18 +211,17 @@ protected slots:
     void on_contents_triggered();
     void on_about_triggered();
 
-    QString currentPage_textFromValue(int value, bool* ok) const;
-    int currentPage_valueFromText(QString text, bool* ok) const;
-
     void on_focusCurrentPage_activated();
     void on_focusScaleFactor_activated();
 
     void on_toggleToolBars_triggered(bool checked);
     void on_toggleMenuBar_triggered(bool checked);
 
-    void on_searchInitiated(const QString& text, bool allTabs);
+    void on_searchInitiated(const QString& text, bool modified);
     void on_highlightAll_clicked(bool checked);
 
+    void on_dock_dockLocationChanged(Qt::DockWidgetArea area);
+
     void on_outline_sectionCountChanged();
     void on_outline_clicked(const QModelIndex& index);
 
@@ -227,7 +234,9 @@ protected slots:
     void on_bookmarks_clicked(const QModelIndex& index);
     void on_bookmarks_contextMenuRequested(const QPoint& pos);
 
-    void on_database_tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, bool rightToLeftMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
+    void on_search_sectionCountChanged();
+    void on_search_visibilityChanged(bool visible);
+    void on_search_clicked(const QModelIndex& index);
 
     void on_saveDatabase_timeout();
 
@@ -243,6 +252,8 @@ private:
     static Settings* s_settings;
     static Database* s_database;
 
+    void prepareStyle();
+
     TabWidget* m_tabWidget;
 
     DocumentView* currentTab() const;
@@ -254,8 +265,6 @@ private:
     int addTab(DocumentView* tab);
     void closeTab(DocumentView* tab);
 
-    QString tabTitle(const DocumentView* tab) const;
-
     bool saveModifications(DocumentView* tab);
 
     void setWindowTitleForCurrentTab();
@@ -263,6 +272,8 @@ private:
 
     BookmarkModel* bookmarkModelForCurrentTab(bool create = false);
 
+    class RestoreTab;
+
     QTimer* m_saveDatabaseTimer;
 
     void prepareDatabase();
@@ -272,6 +283,8 @@ private:
     void scheduleSaveBookmarks();
     void scheduleSavePerFileSettings();
 
+    class TextValueMapper;
+
     MappingSpinBox* m_currentPageSpinBox;
     QWidgetAction* m_currentPageAction;
 
@@ -280,12 +293,14 @@ private:
 
     SearchLineEdit* m_searchLineEdit;
     QCheckBox* m_matchCaseCheckBox;
+    QCheckBox* m_wholeWordsCheckBox;
     QCheckBox* m_highlightAllCheckBox;
 
     void createWidgets();
 
     QAction* m_openAction;
     QAction* m_openInNewTabAction;
+    QAction* m_openCopyInNewTabAction;
     QAction* m_openContainingFolderAction;
     QAction* m_refreshAction;
     QAction* m_saveCopyAction;
@@ -333,6 +348,7 @@ private:
     QAction* m_rotateRightAction;
 
     QAction* m_invertColorsAction;
+    QAction* m_convertToGrayscaleAction;
 
     QAction* m_fontsAction;
 
@@ -387,19 +403,22 @@ private:
     QTableView* m_bookmarksView;
 
     QDockWidget* m_searchDock;
+    QTreeView* m_searchView;
     QWidget* m_searchWidget;
 
     QDockWidget* createDock(const QString& text, const QString& objectName, const QKeySequence& toggleViewShortcut);
 
+    void createSearchDock();
+
     void createDocks();
 
     QMenu* m_fileMenu;
     RecentlyUsedMenu* m_recentlyUsedMenu;
     QMenu* m_editMenu;
     QMenu* m_viewMenu;
-    QMenu* m_tabsMenu;
+    SearchableMenu* m_tabsMenu;
     RecentlyClosedMenu* m_recentlyClosedMenu;
-    QMenu* m_bookmarksMenu;
+    SearchableMenu* m_bookmarksMenu;
     QMenu* m_helpMenu;
 
     bool m_bookmarksMenuIsDirty;
