@@ -1,7 +1,7 @@
 /*
 
 Copyright 2014 S. Razi Alavizadeh
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2015 Adam Reichold
 Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
@@ -31,23 +31,27 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QMenu>
 #include <QMouseEvent>
 #include <QTimer>
+#include <QToolTip>
 #include <QVBoxLayout>
 
+#include "searchmodel.h"
+
 namespace
 {
 
 using namespace qpdfview;
 
-QMetaMethod slotToMethod(const QMetaObject* metaObject, const char* slot)
+inline bool isPrintable(const QString& string)
 {
-    const int index = *slot == '1' ? metaObject->indexOfSlot(slot + 1) : -1;
-
-    if(index < 0)
+    foreach(QChar character, string)
     {
-        qWarning() << "Could not connect slot:" << slot;
+        if(!character.isPrint())
+        {
+            return false;
+        }
     }
 
-    return metaObject->method(index);
+    return true;
 }
 
 } // anonymous
@@ -71,6 +75,106 @@ void GraphicsCompositionModeEffect::draw(QPainter* painter)
     painter->restore();
 }
 
+ProxyStyle::ProxyStyle() : QProxyStyle(),
+    m_scrollableMenus(false)
+{
+}
+
+bool ProxyStyle::scrollableMenus() const
+{
+    return m_scrollableMenus;
+}
+
+void ProxyStyle::setScrollableMenus(bool scrollableMenus)
+{
+    m_scrollableMenus = scrollableMenus;
+}
+
+int ProxyStyle::styleHint(StyleHint hint, const QStyleOption* option, const QWidget* widget, QStyleHintReturn* returnData) const
+{
+    if(m_scrollableMenus && hint == QStyle::SH_Menu_Scrollable)
+    {
+        return 1;
+    }
+
+    return QProxyStyle::styleHint(hint, option, widget, returnData);
+}
+
+SearchableMenu::SearchableMenu(const QString& title, QWidget* parent) : QMenu(title, parent),
+    m_searchable(false),
+    m_text()
+{
+}
+
+bool SearchableMenu::isSearchable() const
+{
+    return m_searchable;
+}
+
+void SearchableMenu::setSearchable(bool searchable)
+{
+    m_searchable = searchable;
+}
+
+void SearchableMenu::hideEvent(QHideEvent* event)
+{
+    QMenu::hideEvent(event);
+
+    if(m_searchable && !event->spontaneous())
+    {
+        m_text = QString();
+
+        foreach(QAction* action, actions())
+        {
+            action->setVisible(true);
+        }
+    }
+}
+
+void SearchableMenu::keyPressEvent(QKeyEvent* event)
+{
+    if(!m_searchable)
+    {
+        QMenu::keyPressEvent(event);
+        return;
+    }
+
+    const QString text = event->text();
+
+    if(!text.isEmpty() && isPrintable(text))
+    {
+        m_text.append(text);
+    }
+    else if(event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete)
+    {
+        m_text.chop(1);
+    }
+    else
+    {
+        QMenu::keyPressEvent(event);
+        return;
+    }
+
+    setActiveAction(0);
+
+    foreach(QAction* action, actions())
+    {
+        if(!action->data().isNull()) // Modify only flagged actions
+        {
+            const bool visible = action->text().contains(m_text, Qt::CaseInsensitive);
+
+            action->setVisible(visible);
+
+            if(visible && activeAction() == 0)
+            {
+                setActiveAction(action);
+            }
+        }
+    }
+
+    QToolTip::showText(mapToGlobal(rect().topLeft()), tr("Search for '%1'...").arg(m_text), this);
+}
+
 TabBar::TabBar(QWidget* parent) : QTabBar(parent)
 {
 }
@@ -106,7 +210,12 @@ void TabBar::mousePressEvent(QMouseEvent* event)
 
     if(event->button() == Qt::MidButton)
     {
-        emit tabCloseRequested(tabAt(event->pos()));
+        const int index = tabAt(event->pos());
+
+        if(index != -1)
+        {
+            emit tabCloseRequested(index);
+        }
     }
 }
 
@@ -209,7 +318,7 @@ void TreeView::expandAll(const QModelIndex& index)
             expand(index);
         }
 
-        for(int row = 0; row < model()->rowCount(index); ++row)
+        for(int row = 0, rowCount = model()->rowCount(index); row < rowCount; ++row)
         {
             expandAll(index.child(row, 0));
         }
@@ -229,7 +338,7 @@ void TreeView::collapseAll(const QModelIndex& index)
             collapse(index);
         }
 
-        for(int row = 0; row < model()->rowCount(index); ++row)
+        for(int row = 0, rowCount = model()->rowCount(index); row < rowCount; ++row)
         {
             collapseAll(index.child(row, 0));
         }
@@ -244,7 +353,7 @@ void TreeView::restoreExpansion(const QModelIndex& index)
 {
     if(index.isValid())
     {
-        const bool expanded = model()->data(index, m_expansionRole).toBool();
+        const bool expanded = index.data(m_expansionRole).toBool();
 
         if(isExpanded(index) != expanded)
         {
@@ -252,7 +361,7 @@ void TreeView::restoreExpansion(const QModelIndex& index)
         }
     }
 
-    for(int row = 0; row < model()->rowCount(index); ++row)
+    for(int row = 0, rowCount = model()->rowCount(index); row < rowCount; ++row)
     {
         restoreExpansion(model()->index(row, 0, index));
     }
@@ -323,24 +432,17 @@ void SpinBox::keyPressEvent(QKeyEvent* event)
     }
 }
 
-MappingSpinBox::MappingSpinBox(QObject* mapper, const char* textFromValue, const char* valueFromText, QWidget* parent) : SpinBox(parent),
+MappingSpinBox::MappingSpinBox(TextValueMapper* mapper, QWidget* parent) : SpinBox(parent),
     m_mapper(mapper)
 {
-    const QMetaObject* metaObject = mapper->metaObject();
-
-    m_textFromValue = slotToMethod(metaObject, textFromValue);
-    m_valueFromText = slotToMethod(metaObject, valueFromText);
 }
 
 QString MappingSpinBox::textFromValue(int val) const
 {
-    QString text;
     bool ok = false;
+    QString text = m_mapper->textFromValue(val, ok);
 
-    const bool mapped = m_textFromValue.invoke(m_mapper, Qt::DirectConnection,
-                                               Q_RETURN_ARG(QString, text), Q_ARG(int, val), Q_ARG(bool*, &ok));
-
-    if(!mapped || !ok)
+    if(!ok)
     {
         text = SpinBox::textFromValue(val);
     }
@@ -350,13 +452,10 @@ QString MappingSpinBox::textFromValue(int val) const
 
 int MappingSpinBox::valueFromText(const QString& text) const
 {
-    int val;
     bool ok = false;
+    int val = m_mapper->valueFromText(text, ok);
 
-    const bool mapped = m_valueFromText.invoke(m_mapper, Qt::DirectConnection,
-                                               Q_RETURN_ARG(int, val), Q_ARG(QString, text), Q_ARG(bool*, &ok));
-
-    if(!mapped || !ok)
+    if(!ok)
     {
         val = SpinBox::valueFromText(text);
     }
@@ -372,7 +471,7 @@ QValidator::State MappingSpinBox::validate(QString& input, int& pos) const
     return QValidator::Acceptable;
 }
 
-int getMappedNumber(QObject* mapper, const char* textFromValue, const char* valueFromText,
+int getMappedNumber(MappingSpinBox::TextValueMapper* mapper,
                     QWidget* parent, const QString& title, const QString& caption,
                     int value, int min, int max, bool* ok, Qt::WindowFlags flags)
 {
@@ -382,7 +481,7 @@ int getMappedNumber(QObject* mapper, const char* textFromValue, const char* valu
     QLabel* label = new QLabel(dialog);
     label->setText(caption);
 
-    MappingSpinBox* mappingSpinBox = new MappingSpinBox(mapper, textFromValue, valueFromText, dialog);
+    MappingSpinBox* mappingSpinBox = new MappingSpinBox(mapper, dialog);
     mappingSpinBox->setRange(min, max);
     mappingSpinBox->setValue(value);
 
@@ -436,8 +535,11 @@ void ProgressLineEdit::paintEvent(QPaintEvent* event)
 
     QPainter painter(this);
 
+    QRect highlightedRect = rect();
+    highlightedRect.setWidth(m_progress * highlightedRect.width() / 100);
+
     painter.setCompositionMode(QPainter::CompositionMode_Multiply);
-    painter.fillRect(0, 0, m_progress * width() / 100, height(), palette().highlight());
+    painter.fillRect(highlightedRect, palette().highlight());
 }
 
 void ProgressLineEdit::keyPressEvent(QKeyEvent* event)
@@ -467,6 +569,11 @@ void SearchLineEdit::startSearch()
     QTimer::singleShot(0, this, SLOT(on_timeout()));
 }
 
+void SearchLineEdit::startTimer()
+{
+    m_timer->start();
+}
+
 void SearchLineEdit::stopTimer()
 {
     m_timer->stop();
