@@ -1,7 +1,7 @@
 /*
 
 Copyright 2014 S. Razi Alavizadeh
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2015 Adam Reichold
 Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
@@ -27,11 +27,14 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QComboBox>
 #include <QGraphicsEffect>
 #include <QLineEdit>
-#include <QMetaMethod>
+#include <QMenu>
 #include <QPainter>
+#include <QProxyStyle>
 #include <QSpinBox>
 #include <QTreeView>
 
+class QTextLayout;
+
 namespace qpdfview
 {
 
@@ -52,6 +55,49 @@ private:
 
 };
 
+// proxy style
+
+class ProxyStyle : public QProxyStyle
+{
+    Q_OBJECT
+
+public:
+    ProxyStyle();
+
+    bool scrollableMenus() const;
+    void setScrollableMenus(bool scrollableMenus);
+
+    int styleHint(StyleHint hint, const QStyleOption* option, const QWidget* widget, QStyleHintReturn* returnData) const;
+
+private:
+    Q_DISABLE_COPY(ProxyStyle)
+
+    bool m_scrollableMenus;
+
+};
+
+// searchable menu
+
+class SearchableMenu : public QMenu
+{
+    Q_OBJECT
+
+public:
+    SearchableMenu(const QString& title, QWidget* parent = 0);
+
+    bool isSearchable() const;
+    void setSearchable(bool searchable);
+
+protected:
+    void hideEvent(QHideEvent* event);
+    void keyPressEvent(QKeyEvent* event);
+
+private:
+    bool m_searchable;
+    QString m_text;
+
+};
+
 // tab bar
 
 class TabBar : public QTabBar
@@ -200,7 +246,15 @@ class MappingSpinBox : public SpinBox
     Q_OBJECT
 
 public:
-    MappingSpinBox(QObject* mapper, const char* textFromValue, const char* valueFromText, QWidget* parent = 0);
+    struct TextValueMapper
+    {
+        virtual ~TextValueMapper() {}
+
+        virtual QString textFromValue(int val, bool& ok) const = 0;
+        virtual int valueFromText(const QString& text, bool& ok) const = 0;
+    };
+
+    MappingSpinBox(TextValueMapper* mapper, QWidget* parent = 0);
 
 protected:
     QString textFromValue(int val) const;
@@ -211,13 +265,11 @@ protected:
 private:
     Q_DISABLE_COPY(MappingSpinBox)
 
-    QObject* m_mapper;
-    QMetaMethod m_textFromValue;
-    QMetaMethod m_valueFromText;
+    QScopedPointer< TextValueMapper > m_mapper;
 
 };
 
-int getMappedNumber(QObject* mapper, const char* textFromValue, const char* valueFromText,
+int getMappedNumber(MappingSpinBox::TextValueMapper* mapper,
                     QWidget* parent, const QString& title, const QString& caption,
                     int value = 0, int min = -2147483647, int max = 2147483647,
                     bool* ok = 0, Qt::WindowFlags flags = 0);
@@ -257,11 +309,14 @@ class SearchLineEdit : public ProgressLineEdit
 public:
     explicit SearchLineEdit(QWidget* parent = 0);
 
+public slots:
     void startSearch();
+
+    void startTimer();
     void stopTimer();
 
 signals:
-    void searchInitiated(const QString& text, bool allTabs = false);
+    void searchInitiated(const QString& text, bool modified = false);
 
 protected slots:
     void on_timeout();
