@@ -115,7 +115,9 @@ namespace Model
         virtual QList< Link* > links() const { return QList< Link* >(); }
 
         virtual QString text(const QRectF& rect) const { Q_UNUSED(rect); return QString(); }
-        virtual QList< QRectF > search(const QString& text, bool matchCase) const { Q_UNUSED(text); Q_UNUSED(matchCase); return QList< QRectF >(); }
+        virtual QString cachedText(const QRectF& rect) const { return text(rect); }
+
+        virtual QList< QRectF > search(const QString& text, bool matchCase, bool wholeWords) const { Q_UNUSED(text); Q_UNUSED(matchCase); Q_UNUSED(wholeWords); return QList< QRectF >(); }
 
         virtual QList< Annotation* > annotations() const { return QList< Annotation* >(); }
 
@@ -149,6 +151,14 @@ namespace Model
 
         virtual void setPaperColor(const QColor& paperColor) { Q_UNUSED(paperColor); }
 
+        enum
+        {
+            PageRole = Qt::UserRole + 1,
+            LeftRole,
+            TopRole,
+            ExpansionRole
+        };
+
         virtual void loadOutline(QStandardItemModel* outlineModel) const { outlineModel->clear(); }
         virtual void loadProperties(QStandardItemModel* propertiesModel) const { propertiesModel->clear(); }
 
