@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2015 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -46,6 +46,9 @@ namespace
 
 using namespace qpdfview;
 
+const int largeTilesThreshold = 8;
+const int veryLargeTilesThreshold = 16;
+
 const qreal proxyPadding = 2.0;
 
 bool modifiersUseMouseButton(Settings* settings, Qt::MouseButton mouseButton)
@@ -60,12 +63,12 @@ namespace qpdfview
 
 Settings* PageItem::s_settings = 0;
 
-PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphicsItem* parent) : QGraphicsObject(parent),
+PageItem::PageItem(Model::Page* page, int index, PaintMode paintMode, QGraphicsItem* parent) : QGraphicsObject(parent),
     m_page(page),
     m_size(page->size()),
     m_cropRect(),
     m_index(index),
-    m_presentationMode(presentationMode),
+    m_paintMode(paintMode),
     m_highlights(),
     m_links(),
     m_annotations(),
@@ -87,17 +90,15 @@ PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphic
 
     setAcceptHoverEvents(true);
 
-    setFlag(QGraphicsItem::ItemUsesExtendedStyleOption, s_settings->pageItem().useTiling());
+    setFlag(QGraphicsItem::ItemUsesExtendedStyleOption, s_settings->pageItem().useTiling() && !thumbnailMode());
     setFlag(QGraphicsItem::ItemClipsToShape, s_settings->pageItem().trimMargins());
 
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
-        TileItem* tile = new TileItem(this);
-
         m_tileItems.resize(1);
         m_tileItems.squeeze();
 
-        m_tileItems.replace(0, tile);
+        m_tileItems.replace(0, new TileItem(this));
     }
 
     QTimer::singleShot(0, this, SLOT(loadInteractiveElements()));
@@ -271,10 +272,20 @@ void PageItem::setInvertColors(bool invertColors)
     }
 }
 
+void PageItem::setConvertToGrayscale(bool convertToGrayscale)
+{
+    if(m_renderParam.convertToGrayscale != convertToGrayscale)
+    {
+        refresh(false);
+
+        m_renderParam.convertToGrayscale = convertToGrayscale;
+    }
+}
+
 
 void PageItem::refresh(bool keepObsoletePixmaps, bool dropCachedPixmaps)
 {
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
         m_tileItems.first()->refresh(keepObsoletePixmaps);
     }
@@ -303,7 +314,7 @@ int PageItem::startRender(bool prefetch)
 {
     int cost = 0;
 
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
         cost += m_tileItems.first()->startRender(prefetch);
     }
@@ -320,16 +331,18 @@ int PageItem::startRender(bool prefetch)
 
 void PageItem::cancelRender()
 {
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
         m_tileItems.first()->cancelRender();
     }
     else
     {
-        foreach(TileItem* tile, m_tileItems)
+        foreach(TileItem* tile, m_exposedTileItems)
         {
             tile->cancelRender();
         }
+
+        m_exposedTileItems.clear();
     }
 }
 
@@ -397,7 +410,7 @@ void PageItem::hoverMoveEvent(QGraphicsSceneHoverEvent* event)
         {
             if(m_normalizedTransform.map(link->boundary).contains(event->pos()))
             {
-                if(link->page != -1 && (link->urlOrFileName.isNull() || !m_presentationMode))
+                if(link->page != -1 && (link->urlOrFileName.isNull() || !presentationMode()))
                 {
                     setCursor(Qt::PointingHandCursor);
 
@@ -412,7 +425,7 @@ void PageItem::hoverMoveEvent(QGraphicsSceneHoverEvent* event)
 
                     return;
                 }
-                else if(!link->urlOrFileName.isNull() && !m_presentationMode)
+                else if(!link->urlOrFileName.isNull() && !presentationMode())
                 {
                     setCursor(Qt::PointingHandCursor);
                     QToolTip::showText(event->screenPos(), tr("Open '%1'.").arg(link->urlOrFileName));
@@ -422,7 +435,7 @@ void PageItem::hoverMoveEvent(QGraphicsSceneHoverEvent* event)
             }
         }
 
-        if(m_presentationMode)
+        if(presentationMode())
         {
             unsetCursor();
             QToolTip::hideText();
@@ -469,27 +482,29 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
 {
     const Qt::KeyboardModifiers copyToClipboardModifiers = s_settings->pageItem().copyToClipboardModifiers();
     const Qt::KeyboardModifiers addAnnotationModifiers = s_settings->pageItem().addAnnotationModifiers();
+    const Qt::KeyboardModifiers zoomToSelectionModifiers = s_settings->pageItem().zoomToSelectionModifiers();
 
     const bool copyToClipboardModifiersActive = event->modifiers() == copyToClipboardModifiers || ((event->buttons() & copyToClipboardModifiers) != 0);
     const bool addAnnotationModifiersActive = event->modifiers() == addAnnotationModifiers || ((event->buttons() & addAnnotationModifiers) != 0);
+    const bool zoomToSelectionModifiersActive = event->modifiers() == zoomToSelectionModifiers || ((event->buttons() & zoomToSelectionModifiers) != 0);
 
     // rubber band
 
-    if(m_rubberBandMode == ModifiersMode && !m_presentationMode
-            && (event->modifiers() == Qt::NoModifier || copyToClipboardModifiersActive || addAnnotationModifiersActive)
-            && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
+    if(m_rubberBandMode == ModifiersMode && !presentationMode()
+            && (copyToClipboardModifiersActive || addAnnotationModifiersActive || zoomToSelectionModifiersActive)
+            && event->button() == Qt::LeftButton)
     {
         setCursor(Qt::CrossCursor);
 
-        if(copyToClipboardModifiersActive && event->button() == Qt::LeftButton)
+        if(copyToClipboardModifiersActive)
         {
             m_rubberBandMode = CopyToClipboardMode;
         }
-        else if(addAnnotationModifiersActive && event->button() == Qt::LeftButton)
+        else if(addAnnotationModifiersActive)
         {
             m_rubberBandMode = AddAnnotationMode;
         }
-        else if(event->modifiers() == Qt::NoModifier && event->button() == Qt::MidButton)
+        else if(zoomToSelectionModifiersActive)
         {
             m_rubberBandMode = ZoomToSelectionMode;
         }
@@ -518,21 +533,23 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
             {
                 unsetCursor();
 
-                if(link->page != -1 && (link->urlOrFileName.isNull() || !m_presentationMode))
+                if(link->page != -1 && (link->urlOrFileName.isNull() || !presentationMode()))
                 {
+                    const bool newTab = event->button() == Qt::MidButton;
+
                     if(link->urlOrFileName.isNull())
                     {
-                        emit linkClicked(event->button() == Qt::MidButton, link->page, link->left, link->top);
+                        emit linkClicked(newTab, link->page, link->left, link->top);
                     }
                     else
                     {
-                        emit linkClicked(link->urlOrFileName, link->page);
+                        emit linkClicked(newTab, link->urlOrFileName, link->page);
                     }
 
                     event->accept();
                     return;
                 }
-                else if(!link->urlOrFileName.isNull() && !m_presentationMode)
+                else if(!link->urlOrFileName.isNull() && !presentationMode())
                 {
                     emit linkClicked(link->urlOrFileName);
 
@@ -542,7 +559,7 @@ void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
             }
         }
 
-        if(event->button() == Qt::MidButton || m_presentationMode)
+        if(event->button() == Qt::MidButton || presentationMode())
         {
             event->ignore();
             return;
@@ -654,7 +671,7 @@ void PageItem::contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
         return;
     }
 
-    if(m_presentationMode)
+    if(presentationMode())
     {
         event->ignore();
         return;
@@ -693,7 +710,7 @@ void PageItem::loadInteractiveElements()
 {
     m_links = m_page->links();
 
-    if(!m_presentationMode)
+    if(!presentationMode())
     {
         m_annotations = m_page->annotations();
 
@@ -717,7 +734,7 @@ void PageItem::updateCropRect()
 {
     QRectF cropRect;
 
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
         cropRect = m_tileItems.first()->cropRect();
     }
@@ -1071,7 +1088,7 @@ void PageItem::prepareGeometry()
 
 void PageItem::prepareTiling()
 {
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
         m_tileItems.first()->setRect(QRect(0, 0, m_boundingRect.width(), m_boundingRect.height()));
 
@@ -1081,8 +1098,18 @@ void PageItem::prepareTiling()
 
     const qreal pageWidth = m_boundingRect.width();
     const qreal pageHeight = m_boundingRect.height();
+    const qreal pageSize = qMax(pageWidth, pageHeight);
 
-    const int tileSize = s_settings->pageItem().tileSize();
+    int tileSize = s_settings->pageItem().tileSize();
+
+    if(tileSize * veryLargeTilesThreshold < pageSize)
+    {
+        tileSize *= 4;
+    }
+    else if(tileSize * largeTilesThreshold < pageSize)
+    {
+        tileSize *= 2;
+    }
 
     int tileWidth = pageWidth < pageHeight ? tileSize * pageWidth / pageHeight : tileSize;
     int tileHeight = pageHeight < pageWidth ? tileSize * pageHeight / pageWidth : tileSize;
@@ -1097,26 +1124,28 @@ void PageItem::prepareTiling()
     const int newCount = columnCount * rowCount;
     const int oldCount = m_tileItems.count();
 
-    for(int index = newCount; index < oldCount; ++index)
-    {
-        m_tileItems.at(index)->deleteAfterRender();
-    }
-
-    m_tileItems.resize(newCount);
-
-    for(int index = oldCount; index < newCount; ++index)
-    {
-        m_tileItems.replace(index, new TileItem(this));
-    }
-
     if(oldCount != newCount)
     {
+        for(int index = newCount; index < oldCount; ++index)
+        {
+            m_tileItems.at(index)->deleteAfterRender();
+        }
+
+        m_tileItems.resize(newCount);
+
+        for(int index = oldCount; index < newCount; ++index)
+        {
+            m_tileItems.replace(index, new TileItem(this));
+        }
+
         foreach(TileItem* tile, m_tileItems)
         {
             tile->dropObsoletePixmap();
         }
     }
 
+    m_exposedTileItems.clear();
+
 
     for(int column = 0; column < columnCount; ++column)
     {
@@ -1133,9 +1162,9 @@ void PageItem::prepareTiling()
     }
 }
 
-void PageItem::paintPage(QPainter* painter, const QRectF& exposedRect) const
+inline void PageItem::paintPage(QPainter* painter, const QRectF& exposedRect) const
 {
-    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
+    if(s_settings->pageItem().decoratePages() && !presentationMode())
     {
         // background
 
@@ -1151,9 +1180,14 @@ void PageItem::paintPage(QPainter* painter, const QRectF& exposedRect) const
 
     // tiles
 
-    if(!s_settings->pageItem().useTiling())
+    if(!s_settings->pageItem().useTiling() || thumbnailMode())
     {
-        m_tileItems.first()->paint(painter, m_boundingRect.topLeft());
+        TileItem* tile = m_tileItems.first();
+
+        if(tile->paint(painter, m_boundingRect.topLeft()))
+        {
+            tile->dropPixmap();
+        }
     }
     else
     {
@@ -1161,41 +1195,57 @@ void PageItem::paintPage(QPainter* painter, const QRectF& exposedRect) const
 
         foreach(TileItem* tile, m_tileItems)
         {
-            if(translatedExposedRect.intersects(tile->rect()))
+            const bool intersects = translatedExposedRect.intersects(tile->rect());
+            const bool contains = m_exposedTileItems.contains(tile);
+
+            if(intersects && !contains)
             {
-                tile->paint(painter, m_boundingRect.topLeft());
+                m_exposedTileItems.insert(tile);
             }
-            else
+            else if(!intersects && contains)
             {
+                m_exposedTileItems.remove(tile);
+
                 tile->cancelRender();
             }
         }
+
+        bool allExposedPainted = true;
+
+        foreach(TileItem* tile, m_exposedTileItems)
+        {
+            if(!tile->paint(painter, m_boundingRect.topLeft()))
+            {
+                allExposedPainted = false;
+            }
+        }
+
+        if(allExposedPainted)
+        {
+            foreach(TileItem* tile, m_exposedTileItems)
+            {
+                tile->dropPixmap();
+            }
+        }
     }
 
-    if(s_settings->pageItem().decoratePages() && !m_presentationMode)
+    if(s_settings->pageItem().decoratePages() && !presentationMode())
     {
         // border
 
-        if(!s_settings->pageItem().trimMargins())
-        {
-            painter->drawRect(m_boundingRect);
-        }
-        else
-        {
-            painter->save();
+        painter->save();
 
-            painter->setClipping(false);
+        painter->setClipping(false);
 
-            painter->drawRect(boundingRect());
+        painter->drawRect(s_settings->pageItem().trimMargins() ? PageItem::boundingRect() : m_boundingRect);
 
-            painter->restore();
-        }
+        painter->restore();
     }
 }
 
-void PageItem::paintLinks(QPainter* painter) const
+inline void PageItem::paintLinks(QPainter* painter) const
 {
-    if(s_settings->pageItem().decorateLinks() && !m_presentationMode && !m_links.isEmpty())
+    if(s_settings->pageItem().decorateLinks() && !presentationMode() && !m_links.isEmpty())
     {
         painter->save();
 
@@ -1211,9 +1261,9 @@ void PageItem::paintLinks(QPainter* painter) const
     }
 }
 
-void PageItem::paintFormFields(QPainter* painter) const
+inline void PageItem::paintFormFields(QPainter* painter) const
 {
-    if(s_settings->pageItem().decorateFormFields() && !m_presentationMode && !m_formFields.isEmpty())
+    if(s_settings->pageItem().decorateFormFields() && !presentationMode() && !m_formFields.isEmpty())
     {
         painter->save();
 
@@ -1229,7 +1279,7 @@ void PageItem::paintFormFields(QPainter* painter) const
     }
 }
 
-void PageItem::paintHighlights(QPainter* painter) const
+inline void PageItem::paintHighlights(QPainter* painter) const
 {
     if(!m_highlights.isEmpty())
     {
@@ -1249,7 +1299,7 @@ void PageItem::paintHighlights(QPainter* painter) const
     }
 }
 
-void PageItem::paintRubberBand(QPainter* painter) const
+inline void PageItem::paintRubberBand(QPainter* painter) const
 {
     if(!m_rubberBand.isNull())
     {
