@@ -25,6 +25,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QCache>
 #include <QGraphicsObject>
 #include <QIcon>
+#include <QSet>
 
 class QGraphicsProxyWidget;
 
@@ -52,54 +53,63 @@ class PageItem : public QGraphicsObject
     friend class TileItem;
 
 public:
-    PageItem(Model::Page* page, int index, bool presentationMode = false, QGraphicsItem* parent = 0);
+    enum PaintMode
+    {
+        DefaultMode,
+        PresentationMode,
+        ThumbnailMode
+    };
+
+    PageItem(Model::Page* page, int index, PaintMode paintMode = DefaultMode, QGraphicsItem* parent = 0);
     ~PageItem();
 
     QRectF boundingRect() const;
     void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget*);
 
-    inline int index() const { return m_index; }
-    inline bool presentationMode() const { return m_presentationMode; }
+    int index() const { return m_index; }
 
-    inline const QSizeF& size() const { return m_size; }
+    const QSizeF& size() const { return m_size; }
 
     qreal displayedWidth() const;
     qreal displayedHeight() const;
 
-    inline const QList< QRectF >& highlights() const { return m_highlights; }
+    const QList< QRectF >& highlights() const { return m_highlights; }
     void setHighlights(const QList< QRectF >& highlights);
 
-    inline RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
+    RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
     void setRubberBandMode(RubberBandMode rubberBandMode);
 
-    inline bool showsAnnotationOverlay() const { return !m_annotationOverlay.isEmpty(); }
-    inline bool showsFormFieldOverlay() const { return !m_formFieldOverlay.isEmpty(); }
+    bool showsAnnotationOverlay() const { return !m_annotationOverlay.isEmpty(); }
+    bool showsFormFieldOverlay() const { return !m_formFieldOverlay.isEmpty(); }
 
-    inline int resolutionX() const { return m_renderParam.resolution.resolutionX; }
-    inline int resolutionY() const { return m_renderParam.resolution.resolutionY; }
+    int resolutionX() const { return m_renderParam.resolution.resolutionX; }
+    int resolutionY() const { return m_renderParam.resolution.resolutionY; }
     void setResolution(int resolutionX, int resolutionY);
 
-    inline qreal devicePixelRatio() const { return m_renderParam.resolution.devicePixelRatio; }
+    qreal devicePixelRatio() const { return m_renderParam.resolution.devicePixelRatio; }
     void setDevicePixelRatio(qreal devicePixelRatio);
 
-    inline qreal scaleFactor() const { return m_renderParam.scaleFactor; }
+    qreal scaleFactor() const { return m_renderParam.scaleFactor; }
     void setScaleFactor(qreal scaleFactor);
 
-    inline Rotation rotation() const { return m_renderParam.rotation; }
+    Rotation rotation() const { return m_renderParam.rotation; }
     void setRotation(Rotation rotation);
 
-    inline bool invertColors() { return m_renderParam.invertColors; }
+    bool invertColors() { return m_renderParam.invertColors; }
     void setInvertColors(bool invertColors);
 
-    inline const QTransform& transform() const { return m_transform; }
-    inline const QTransform& normalizedTransform() const { return m_normalizedTransform; }
+    bool convertToGrayscale() { return m_renderParam.convertToGrayscale; }
+    void setConvertToGrayscale(bool convertToGrayscale);
+
+    const QTransform& transform() const { return m_transform; }
+    const QTransform& normalizedTransform() const { return m_normalizedTransform; }
 
 signals:
     void cropRectChanged();
 
     void linkClicked(bool newTab, int page, qreal left = 0.0, qreal top = 0.0);
+    void linkClicked(bool newTab, const QString& fileName, int page);
     void linkClicked(const QString& url);
-    void linkClicked(const QString& fileName, int page);
 
     void rubberBandStarted();
     void rubberBandFinished();
@@ -152,7 +162,10 @@ private:
     void updateCropRect();
 
     int m_index;
-    bool m_presentationMode;
+    PaintMode m_paintMode;
+
+    bool presentationMode() const { return m_paintMode == PresentationMode; }
+    bool thumbnailMode() const { return m_paintMode == ThumbnailMode; }
 
     QList< QRectF > m_highlights;
 
@@ -198,6 +211,7 @@ private:
     void prepareGeometry();
 
     QVector< TileItem* > m_tileItems;
+    mutable QSet< TileItem* > m_exposedTileItems;
 
     void prepareTiling();
 
