@@ -22,6 +22,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "pdfmodel.h"
 
+#include <QCache>
 #include <QFormLayout>
 #include <QMessageBox>
 #include <QSettings>
@@ -41,9 +42,28 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "annotationwidgets.h"
 #include "formfieldwidgets.h"
 
+#ifndef HAS_POPPLER_24
+
+#define LOCK_ANNOTATION QMutexLocker mutexLocker(m_mutex);
+#define LOCK_FORM_FIELD QMutexLocker mutexLocker(m_mutex);
+#define LOCK_PAGE QMutexLocker mutexLocker(m_mutex);
+#define LOCK_DOCUMENT QMutexLocker mutexLocker(&m_mutex);
+
+#else
+
+#define LOCK_ANNOTATION
+#define LOCK_FORM_FIELD
+#define LOCK_PAGE
+#define LOCK_DOCUMENT
+
+#endif // HAS_POPPLER_24
+
 namespace
 {
 
+using namespace qpdfview;
+using namespace qpdfview::Model;
+
 void loadOutline(Poppler::Document* document, const QDomNode& node, QStandardItem* parent)
 {
     const QDomElement element = node.toElement();
@@ -89,9 +109,9 @@ void loadOutline(Poppler::Document* document, const QDomNode& node, QStandardIte
 
         delete linkDestination;
 
-        item->setData(page, Qt::UserRole + 1);
-        item->setData(left, Qt::UserRole + 2);
-        item->setData(top, Qt::UserRole + 3);
+        item->setData(page, Document::PageRole);
+        item->setData(left, Document::LeftRole);
+        item->setData(top, Document::TopRole);
 
         QStandardItem* pageItem = item->clone();
         pageItem->setText(QString::number(page));
@@ -117,6 +137,19 @@ void loadOutline(Poppler::Document* document, const QDomNode& node, QStandardIte
     }
 }
 
+inline void restoreRenderHint(Poppler::Document* document, const Poppler::Document::RenderHints hints, const Poppler::Document::RenderHint hint)
+{
+    document->setRenderHint(hint, hints.testFlag(hint));
+}
+
+typedef QSharedPointer< Poppler::TextBox > TextBox;
+typedef QList< TextBox > TextBoxList;
+
+QCache< const PdfPage*,  TextBoxList > textCache(1 << 12);
+QMutex textCacheMutex;
+
+#define LOCK_TEXT_CACHE QMutexLocker mutexLocker(&textCacheMutex);
+
 } // anonymous
 
 namespace qpdfview
@@ -138,22 +171,14 @@ PdfAnnotation::~PdfAnnotation()
 
 QRectF PdfAnnotation::boundary() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_ANNOTATION
 
     return m_annotation->boundary().normalized();
 }
 
 QString PdfAnnotation::contents() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_ANNOTATION
 
     return m_annotation->contents();
 }
@@ -191,22 +216,14 @@ PdfFormField::~PdfFormField()
 
 QRectF PdfFormField::boundary() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_FORM_FIELD
 
     return m_formField->rect().normalized();
 }
 
 QString PdfFormField::name() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_FORM_FIELD
 
     return m_formField->name();
 }
@@ -268,27 +285,25 @@ PdfPage::PdfPage(QMutex* mutex, Poppler::Page* page) :
 
 PdfPage::~PdfPage()
 {
+    {
+        LOCK_TEXT_CACHE
+
+        textCache.remove(this);
+    }
+
     delete m_page;
 }
 
 QSizeF PdfPage::size() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     return m_page->pageSizeF();
 }
 
 QImage PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     Poppler::Page::Rotation rotate;
 
@@ -327,22 +342,14 @@ QImage PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rot
 
 QString PdfPage::label() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     return m_page->label();
 }
 
 QList< Link* > PdfPage::links() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     QList< Link* > links;
 
@@ -397,65 +404,119 @@ QList< Link* > PdfPage::links() const
 
 QString PdfPage::text(const QRectF& rect) const
 {
-#ifndef HAS_POPPLER_24
+    LOCK_PAGE
 
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
-
-    return m_page->text(rect);
+    return m_page->text(rect).simplified();
 }
 
-QList< QRectF > PdfPage::search(const QString& text, bool matchCase) const
+QString PdfPage::cachedText(const QRectF& rect) const
 {
-#ifndef HAS_POPPLER_24
+    bool wasCached = false;
+    TextBoxList textBoxes;
 
-    QMutexLocker mutexLocker(m_mutex);
+    {
+        LOCK_TEXT_CACHE
 
-#endif // HAS_POPPLER_24
+        if(const TextBoxList* object = textCache.object(this))
+        {
+            wasCached = true;
+
+            textBoxes = *object;
+        }
+    }
+
+    if(!wasCached)
+    {
+        {
+            LOCK_PAGE
+
+            foreach(Poppler::TextBox* textBox, m_page->textList())
+            {
+                textBoxes.append(TextBox(textBox));
+            }
+        }
+
+        LOCK_TEXT_CACHE
+
+        textCache.insert(this, new TextBoxList(textBoxes), textBoxes.count());
+    }
+
+    QString text;
+
+    foreach(const TextBox& textBox, textBoxes)
+    {
+        if(!rect.intersects(textBox->boundingBox()))
+        {
+            continue;
+        }
+
+        const QString& characters = textBox->text();
+
+        for(int index = 0; index < characters.length(); ++index)
+        {
+            if(rect.intersects(textBox->charBoundingBox(index)))
+            {
+                text.append(characters.at(index));
+            }
+        }
+
+        if(textBox->hasSpaceAfter())
+        {
+            text.append(QLatin1Char(' '));
+        }
+    }
+
+    return text.simplified();
+}
+
+QList< QRectF > PdfPage::search(const QString& text, bool matchCase, bool wholeWords) const
+{
+    LOCK_PAGE
 
     QList< QRectF > results;
 
-#if defined(HAS_POPPLER_22)
+#if defined(HAS_POPPLER_31)
 
-    results = m_page->search(text, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive);
+    const Poppler::Page::SearchFlags flags((matchCase ? 0 : Poppler::Page::IgnoreCase) | (wholeWords ? Poppler::Page::WholeWords : 0));
+
+    results = m_page->search(text, flags);
+
+#elif defined(HAS_POPPLER_22)
+
+    const Poppler::Page::SearchMode mode = matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive;
+
+    results = m_page->search(text, mode);
 
 #elif defined(HAS_POPPLER_14)
 
+    const Poppler::Page::SearchMode mode = matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive;
+
     double left = 0.0, top = 0.0, right = 0.0, bottom = 0.0;
 
-    while(m_page->search(text, left, top, right, bottom, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
+    while(m_page->search(text, left, top, right, bottom, Poppler::Page::NextResult, mode))
     {
-        QRectF rect;
-        rect.setLeft(left);
-        rect.setTop(top);
-        rect.setRight(right);
-        rect.setBottom(bottom);
-
-        results.append(rect);
+        results.append(QRectF(left, top, right - left, bottom - top));
     }
 
 #else
 
+    const Poppler::Page::SearchMode mode = matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive;
+
     QRectF rect;
 
-    while(m_page->search(text, rect, Poppler::Page::NextResult, matchCase ? Poppler::Page::CaseSensitive : Poppler::Page::CaseInsensitive))
+    while(m_page->search(text, rect, Poppler::Page::NextResult, mode))
     {
         results.append(rect);
     }
 
-#endif // HAS_POPPLER_22 HAS_POPPLER_14
+#endif // HAS_POPPLER_31 HAS_POPPLER_22 HAS_POPPLER_14
 
     return results;
 }
 
 QList< Annotation* > PdfPage::annotations() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     QList< Annotation* > annotations;
 
@@ -490,11 +551,7 @@ bool PdfPage::canAddAndRemoveAnnotations() const
 
 Annotation* PdfPage::addTextAnnotation(const QRectF& boundary, const QColor& color)
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
 #ifdef HAS_POPPLER_20
 
@@ -526,11 +583,7 @@ Annotation* PdfPage::addTextAnnotation(const QRectF& boundary, const QColor& col
 
 Annotation* PdfPage::addHighlightAnnotation(const QRectF& boundary, const QColor& color)
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
 #ifdef HAS_POPPLER_20
 
@@ -571,11 +624,7 @@ Annotation* PdfPage::addHighlightAnnotation(const QRectF& boundary, const QColor
 
 void PdfPage::removeAnnotation(Annotation* annotation)
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
 #ifdef HAS_POPPLER_20
 
@@ -593,11 +642,7 @@ void PdfPage::removeAnnotation(Annotation* annotation)
 
 QList< FormField* > PdfPage::formFields() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_PAGE
 
     QList< FormField* > formFields;
 
@@ -659,22 +704,14 @@ PdfDocument::~PdfDocument()
 
 int PdfDocument::numberOfPages() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     return m_document->numPages();
 }
 
 Page* PdfDocument::page(int index) const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     Poppler::Page* page = m_document->page(index);
 
@@ -683,24 +720,53 @@ Page* PdfDocument::page(int index) const
 
 bool PdfDocument::isLocked() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     return m_document->isLocked();
 }
 
 bool PdfDocument::unlock(const QString& password)
 {
-#ifndef HAS_POPPLER_24
+    LOCK_DOCUMENT
 
-    QMutexLocker mutexLocker(&m_mutex);
+    // Poppler drops render hints and backend after unlocking so we need to restore them.
+
+    const Poppler::Document::RenderHints hints = m_document->renderHints();
+    const Poppler::Document::RenderBackend backend = m_document->renderBackend();
+
+    const bool ok = m_document->unlock(password.toLatin1(), password.toLatin1());
+
+    restoreRenderHint(m_document, hints, Poppler::Document::Antialiasing);
+    restoreRenderHint(m_document, hints, Poppler::Document::TextAntialiasing);
+
+#ifdef HAS_POPPLER_14
+
+    restoreRenderHint(m_document, hints, Poppler::Document::TextHinting);
+
+#endif // HAS_POPPLER_18 HAS_POPPLER_14
+
+#ifdef HAS_POPPLER_18
+
+    restoreRenderHint(m_document, hints, Poppler::Document::TextSlightHinting);
+
+#endif // HAS_POPPLER_18
+
+#ifdef HAS_POPPLER_22
+
+    restoreRenderHint(m_document, hints, Poppler::Document::OverprintPreview);
+
+#endif // HAS_POPPLER_22
+
+#ifdef HAS_POPPLER_24
+
+    restoreRenderHint(m_document, hints, Poppler::Document::ThinLineSolid);
+    restoreRenderHint(m_document, hints, Poppler::Document::ThinLineShape);
 
 #endif // HAS_POPPLER_24
 
-    return m_document->unlock(password.toLatin1(), password.toLatin1());
+    m_document->setRenderBackend(backend);
+
+    return ok;
 }
 
 QStringList PdfDocument::saveFilter() const
@@ -715,11 +781,7 @@ bool PdfDocument::canSave() const
 
 bool PdfDocument::save(const QString& filePath, bool withChanges) const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     QScopedPointer< Poppler::PDFConverter > pdfConverter(m_document->pdfConverter());
 
@@ -740,24 +802,16 @@ bool PdfDocument::canBePrintedUsingCUPS() const
 
 void PdfDocument::setPaperColor(const QColor& paperColor)
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     m_document->setPaperColor(paperColor);
 }
 
-void Model::PdfDocument::loadOutline(QStandardItemModel* outlineModel) const
+void PdfDocument::loadOutline(QStandardItemModel* outlineModel) const
 {
     Document::loadOutline(outlineModel);
 
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     QDomDocument* toc = m_document->toc();
 
@@ -773,11 +827,7 @@ void PdfDocument::loadProperties(QStandardItemModel* propertiesModel) const
 {
     Document::loadProperties(propertiesModel);
 
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     QStringList keys = m_document->infoKeys();
 
@@ -803,11 +853,7 @@ void PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
 {
     Document::loadFonts(fontsModel);
 
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     const QList< Poppler::FontInfo > fonts = m_document->fonts();
 
@@ -830,11 +876,7 @@ void PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
 
 bool PdfDocument::wantsContinuousMode() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
 
@@ -845,11 +887,7 @@ bool PdfDocument::wantsContinuousMode() const
 
 bool PdfDocument::wantsSinglePageMode() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
 
@@ -859,11 +897,7 @@ bool PdfDocument::wantsSinglePageMode() const
 
 bool PdfDocument::wantsTwoPagesMode() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
 
@@ -873,11 +907,7 @@ bool PdfDocument::wantsTwoPagesMode() const
 
 bool PdfDocument::wantsTwoPagesWithCoverPageMode() const
 {
-#ifndef HAS_POPPLER_24
-
-    QMutexLocker mutexLocker(&m_mutex);
-
-#endif // HAS_POPPLER_24
+    LOCK_DOCUMENT
 
     const Poppler::Document::PageLayout pageLayout = m_document->pageLayout();
 
