@@ -115,7 +115,9 @@ namespace Model
         QList< Link* > links() const;
 
         QString text(const QRectF& rect) const;
-        QList< QRectF > search(const QString& text, bool matchCase) const;
+        QString cachedText(const QRectF& rect) const;
+
+        QList< QRectF > search(const QString& text, bool matchCase, bool wholeWords) const;
 
         QList< Annotation* > annotations() const;
 
