@@ -30,20 +30,26 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <libspectre/spectre-document.h>
 
-Model::PsPage::PsPage(QMutex* mutex, SpectrePage* page, SpectreRenderContext* renderContext) :
+namespace qpdfview
+{
+
+namespace Model
+{
+
+PsPage::PsPage(QMutex* mutex, SpectrePage* page, SpectreRenderContext* renderContext) :
     m_mutex(mutex),
     m_page(page),
     m_renderContext(renderContext)
 {
 }
 
-Model::PsPage::~PsPage()
+PsPage::~PsPage()
 {
     spectre_page_free(m_page);
     m_page = 0;
 }
 
-QSizeF Model::PsPage::size() const
+QSizeF PsPage::size() const
 {
     QMutexLocker mutexLocker(m_mutex);
 
@@ -55,7 +61,7 @@ QSizeF Model::PsPage::size() const
     return QSizeF(w, h);
 }
 
-QImage Model::PsPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
+QImage PsPage::render(qreal horizontalResolution, qreal verticalResolution, Rotation rotation, const QRect& boundingRect) const
 {
     QMutexLocker mutexLocker(m_mutex);
 
@@ -131,14 +137,14 @@ QImage Model::PsPage::render(qreal horizontalResolution, qreal verticalResolutio
     return image;
 }
 
-Model::PsDocument::PsDocument(SpectreDocument* document, SpectreRenderContext* renderContext) :
+PsDocument::PsDocument(SpectreDocument* document, SpectreRenderContext* renderContext) :
     m_mutex(),
     m_document(document),
     m_renderContext(renderContext)
 {
 }
 
-Model::PsDocument::~PsDocument()
+PsDocument::~PsDocument()
 {
     spectre_render_context_free(m_renderContext);
     m_renderContext = 0;
@@ -147,14 +153,14 @@ Model::PsDocument::~PsDocument()
     m_document = 0;
 }
 
-int Model::PsDocument::numberOfPages() const
+int PsDocument::numberOfPages() const
 {
     QMutexLocker mutexLocker(&m_mutex);
 
     return spectre_document_get_n_pages(m_document);
 }
 
-Model::Page* Model::PsDocument::page(int index) const
+Page* PsDocument::page(int index) const
 {
     QMutexLocker mutexLocker(&m_mutex);
 
@@ -163,7 +169,7 @@ Model::Page* Model::PsDocument::page(int index) const
     return page != 0 ? new PsPage(&m_mutex, page, m_renderContext) : 0;
 }
 
-QStringList Model::PsDocument::saveFilter() const
+QStringList PsDocument::saveFilter() const
 {
     QMutexLocker mutexLocker(&m_mutex);
 
@@ -177,12 +183,12 @@ QStringList Model::PsDocument::saveFilter() const
     }
 }
 
-bool Model::PsDocument::canSave() const
+bool PsDocument::canSave() const
 {
     return true;
 }
 
-bool Model::PsDocument::save(const QString& filePath, bool withChanges) const
+bool PsDocument::save(const QString& filePath, bool withChanges) const
 {
     Q_UNUSED(withChanges)
 
@@ -193,12 +199,12 @@ bool Model::PsDocument::save(const QString& filePath, bool withChanges) const
     return (spectre_document_status(m_document) == SPECTRE_STATUS_SUCCESS);
 }
 
-bool Model::PsDocument::canBePrintedUsingCUPS() const
+bool PsDocument::canBePrintedUsingCUPS() const
 {
     return true;
 }
 
-void Model::PsDocument::loadProperties(QStandardItemModel* propertiesModel) const
+void PsDocument::loadProperties(QStandardItemModel* propertiesModel) const
 {
     Document::loadProperties(propertiesModel);
 
@@ -221,6 +227,8 @@ void Model::PsDocument::loadProperties(QStandardItemModel* propertiesModel) cons
     propertiesModel->appendRow(QList< QStandardItem* >() << new QStandardItem(tr("Language level")) << new QStandardItem(languageLevel));
 }
 
+} // Model
+
 PsSettingsWidget::PsSettingsWidget(QSettings* settings, QWidget* parent) : SettingsWidget(parent),
     m_settings(settings)
 {
@@ -289,8 +297,10 @@ SettingsWidget* PsPlugin::createSettingsWidget(QWidget* parent) const
     return new PsSettingsWidget(m_settings, parent);
 }
 
+} // qpdfview
+
 #if QT_VERSION < QT_VERSION_CHECK(5,0,0)
 
-Q_EXPORT_PLUGIN2(qpdfview_ps, PsPlugin)
+Q_EXPORT_PLUGIN2(qpdfview_ps, qpdfview::PsPlugin)
 
 #endif // QT_VERSION
