@@ -36,6 +36,9 @@ struct SpectreRenderContext;
 
 #include "model.h"
 
+namespace qpdfview
+{
+
 class PsPlugin;
 
 namespace Model
@@ -66,7 +69,7 @@ namespace Model
     {
         Q_DECLARE_TR_FUNCTIONS(Model::PsDocument)
 
-        friend class ::PsPlugin;
+        friend class qpdfview::PsPlugin;
 
     public:
         ~PsDocument();
@@ -121,7 +124,7 @@ private:
 class PsPlugin : public QObject, Plugin
 {
     Q_OBJECT
-    Q_INTERFACES(Plugin)
+    Q_INTERFACES(qpdfview::Plugin)
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
 
@@ -143,4 +146,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // PSMODEL_H
