@@ -24,14 +24,19 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMenu>
 
+class QFileInfo;
+
+namespace qpdfview
+{
+
 class RecentlyUsedMenu : public QMenu
 {
     Q_OBJECT
 
 public:
-    explicit RecentlyUsedMenu(int count, QWidget* parent = 0);
+    explicit RecentlyUsedMenu(const QStringList& filePaths, int count, QWidget* parent = 0);
 
-    void addOpenAction(const QString& filePath);
+    void addOpenAction(const QFileInfo& fileInfo);
     void removeOpenAction(const QString& filePath);
 
     QStringList filePaths() const;
@@ -54,4 +59,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // RECENTLYUSEDMENU_H
