@@ -1,6 +1,6 @@
 /*
 
-Copyright 2013 Adam Reichold
+Copyright 2013-2015 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -31,69 +31,6 @@ namespace
 
 using namespace qpdfview;
 
-enum
-{
-    NotCanceled = 0,
-    CanceledNormally = 1,
-    CanceledForcibly = 2
-};
-
-void setCancellation(QAtomicInt& wasCanceled, bool force)
-{
-#if QT_VERSION > QT_VERSION_CHECK(5,0,0)
-
-    wasCanceled.storeRelease(force ? CanceledForcibly : CanceledNormally);
-
-#else
-
-    wasCanceled.fetchAndStoreRelease(force ? CanceledForcibly : CanceledNormally);
-
-#endif // QT_VERSION
-}
-
-void resetCancellation(QAtomicInt& wasCanceled)
-{
-#if QT_VERSION > QT_VERSION_CHECK(5,0,0)
-
-    wasCanceled.storeRelease(NotCanceled);
-
-#else
-
-    wasCanceled.fetchAndStoreRelease(NotCanceled);
-
-#endif // QT_VERSION
-}
-
-bool testCancellation(QAtomicInt& wasCanceled, bool prefetch)
-{
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-
-    return prefetch ?
-                wasCanceled.load() == CanceledForcibly :
-                wasCanceled.load() != NotCanceled;
-
-#else
-
-    return prefetch ?
-                wasCanceled.testAndSetRelaxed(CanceledForcibly, CanceledForcibly) :
-                !wasCanceled.testAndSetRelaxed(NotCanceled, NotCanceled);
-
-#endif // QT_VERSION
-}
-
-int loadWasCanceled(const QAtomicInt& wasCanceled)
-{
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-
-    return wasCanceled.load();
-
-#else
-
-    return wasCanceled;
-
-#endif // QT_VERSION
-}
-
 qreal scaledResolutionX(const RenderParam& renderParam)
 {
     return renderParam.resolution.devicePixelRatio *
@@ -198,6 +135,20 @@ QRectF trimMargins(QRgb paperColor, const QImage& image)
                   static_cast< qreal >(bottom - top) / height);
 }
 
+void convertToGrayscale(QImage& image)
+{
+    QRgb* const begin = reinterpret_cast< QRgb* >(image.bits());
+    QRgb* const end = reinterpret_cast< QRgb* >(image.bits() + image.byteCount());
+
+    for(QRgb* pointer = begin; pointer != end; ++pointer)
+    {
+        const int gray = qGray(*pointer);
+        const int alpha = qAlpha(*pointer);
+
+        *pointer = qRgba(gray, gray, gray, alpha);
+    }
+}
+
 } // anonymous
 
 namespace qpdfview
@@ -233,24 +184,9 @@ bool RenderTask::isRunning() const
     return m_isRunning;
 }
 
-bool RenderTask::wasCanceled() const
-{
-    return loadWasCanceled(m_wasCanceled) != NotCanceled;
-}
-
-bool RenderTask::wasCanceledNormally() const
-{
-    return loadWasCanceled(m_wasCanceled) == CanceledNormally;
-}
-
-bool RenderTask::wasCanceledForcibly() const
-{
-    return loadWasCanceled(m_wasCanceled) == CanceledForcibly;
-}
-
 void RenderTask::run()
 {
-#define CANCELLATION_POINT if(testCancellation(m_wasCanceled, m_prefetch)) { finish(); return; }
+#define CANCELLATION_POINT if(testCancellation()) { finish(); return; }
 
     CANCELLATION_POINT
 
@@ -266,13 +202,6 @@ void RenderTask::run()
 
 #endif // QT_VERSION
 
-    if(m_renderParam.invertColors)
-    {
-        CANCELLATION_POINT
-
-        image.invertPixels();
-    }
-
     if(m_trimMargins)
     {
         CANCELLATION_POINT
@@ -280,6 +209,20 @@ void RenderTask::run()
         cropRect = trimMargins(m_paperColor.rgb(), image);
     }
 
+    if(m_renderParam.convertToGrayscale)
+    {
+        CANCELLATION_POINT
+
+        convertToGrayscale(image);
+    }
+
+    if(m_renderParam.invertColors)
+    {
+        CANCELLATION_POINT
+
+        image.invertPixels();
+    }
+
     CANCELLATION_POINT
 
     emit imageReady(m_renderParam,
@@ -307,16 +250,11 @@ void RenderTask::start(const RenderParam& renderParam,
     m_isRunning = true;
     m_mutex.unlock();
 
-    resetCancellation(m_wasCanceled);
+    resetCancellation();
 
     QThreadPool::globalInstance()->start(this, prefetch ? 0 : 1);
 }
 
-void RenderTask::cancel(bool force)
-{
-    setCancellation(m_wasCanceled, force);
-}
-
 void RenderTask::finish()
 {
     emit finished();
