@@ -200,6 +200,16 @@ void Settings::PageItem::setAddAnnotationModifiers(const Qt::KeyboardModifiers&
     m_settings->setValue("pageItem/addAnnotationModifiers", static_cast< int >(modifiers));
 }
 
+Qt::KeyboardModifiers Settings::PageItem::zoomToSelectionModifiers() const
+{
+    return static_cast< Qt::KeyboardModifiers>(m_settings->value("pageItem/zoomToSelectionModifiers", static_cast< int >(Defaults::PageItem::zoomToSelectionModifiers())).toInt());
+}
+
+void Settings::PageItem::setZoomToSelectionModifiers(const Qt::KeyboardModifiers& modifiers)
+{
+    m_settings->setValue("pageItem/zoomToSelectionModifiers", static_cast< int >(modifiers));
+}
+
 bool Settings::PageItem::annotationOverlay() const
 {
     return m_settings->value("pageItem/annotationOverlay", Defaults::PageItem::annotationOverlay()).toBool();
@@ -422,6 +432,16 @@ void Settings::DocumentView::setMatchCase(bool matchCase)
     m_settings->setValue("documentView/matchCase", matchCase);
 }
 
+bool Settings::DocumentView::wholeWords() const
+{
+    return m_settings->value("documentView/wholeWords", Defaults::DocumentView::wholeWords()).toBool();
+}
+
+void Settings::DocumentView::setWholeWords(bool wholeWords)
+{
+    m_settings->setValue("documentView/wholeWords", wholeWords);
+}
+
 int Settings::DocumentView::highlightDuration() const
 {
     return m_settings->value("documentView/highlightDuration", Defaults::DocumentView::highlightDuration()).toInt();
@@ -547,6 +567,16 @@ void Settings::DocumentView::setInvertColors(bool invertColors)
     m_settings->setValue("documentView/invertColors", invertColors);
 }
 
+bool Settings::DocumentView::convertToGrayscale() const
+{
+    return m_settings->value("documentView/convertToGrayscale", Defaults::DocumentView::convertToGrayscale()).toBool();
+}
+
+void Settings::DocumentView::setConvertToGrayscale(bool convertToGrayscale)
+{
+    m_settings->setValue("documentView/convertToGrayscale", convertToGrayscale);
+}
+
 bool Settings::DocumentView::highlightAll() const
 {
     return m_settings->value("documentView/highlightAll", Defaults::DocumentView::highlightAll()).toBool();
@@ -708,6 +738,16 @@ void Settings::MainWindow::setNewTabNextToCurrentTab(bool newTabNextToCurrentTab
     m_settings->setValue("mainWindow/newTabNextToCurrentTab", newTabNextToCurrentTab);
 }
 
+bool Settings::MainWindow::exitAfterLastTab() const
+{
+    return m_settings->value("mainWindow/exitAfterLastTab", Defaults::MainWindow::exitAfterLastTab()).toBool();
+}
+
+void Settings::MainWindow::setExitAfterLastTab(bool exitAfterLastTab)
+{
+    m_settings->setValue("mainWindow/exitAfterLastTab", exitAfterLastTab);
+}
+
 bool Settings::MainWindow::documentTitleAsTabTitle() const
 {
     return m_settings->value("mainWindow/documentAsTabTitle", Defaults::MainWindow::documentTitleAsTabTitle()).toBool();
@@ -738,6 +778,16 @@ void Settings::MainWindow::setInstanceNameInWindowTitle(bool instanceNameInWindo
     m_settings->setValue("mainWindow/instanceNameInWindowTitle", instanceNameInWindowTitle);
 }
 
+bool Settings::MainWindow::extendedSearchDock() const
+{
+    return m_settings->value("mainWindow/extendedSearchDock", Defaults::MainWindow::extendedSearchDock()).toBool();
+}
+
+void Settings::MainWindow::setExtendedSearchDock(bool extendedSearchDock)
+{
+    return m_settings->setValue("mainWindow/extendedSearchDock", extendedSearchDock);
+}
+
 bool Settings::MainWindow::usePageLabel() const
 {
     return m_settings->value("mainWindow/usePageLabel", Defaults::MainWindow::usePageLabel()).toBool();
@@ -788,6 +838,26 @@ void Settings::MainWindow::setViewToolBar(const QStringList& viewToolBar)
     m_settings->setValue("mainWindow/viewToolBar", trimmed(viewToolBar));
 }
 
+bool Settings::MainWindow::scrollableMenus() const
+{
+    return m_settings->value("mainWindow/scrollableMenus", Defaults::MainWindow::scrollableMenus()).toBool();
+}
+
+void Settings::MainWindow::setScrollableMenus(bool scrollableMenus)
+{
+    m_settings->setValue("mainWindow/scrollableMenus", scrollableMenus);
+}
+
+bool Settings::MainWindow::searchableMenus() const
+{
+    return m_settings->value("mainWindow/searchableMenus", Defaults::MainWindow::searchableMenus()).toBool();
+}
+
+void Settings::MainWindow::setSearchableMenus(bool searchableMenus)
+{
+    m_settings->setValue("mainWindow/searchableMenus", searchableMenus);
+}
+
 bool Settings::MainWindow::toggleToolAndMenuBarsWithFullscreen() const
 {
     return m_settings->value("mainWindow/toggleToolAndMenuBarsWithFullscreen", Defaults::MainWindow::toggleToolAndMenuBarsWithFullscreen()).toBool();
@@ -1002,31 +1072,6 @@ void Settings::sync()
     m_documentView.sync();
 }
 
-Settings::PageItem& Settings::pageItem()
-{
-    return m_pageItem;
-}
-
-Settings::PresentationView& Settings::presentationView()
-{
-    return m_presentationView;
-}
-
-Settings::DocumentView& Settings::documentView()
-{
-    return m_documentView;
-}
-
-Settings::MainWindow& Settings::mainWindow()
-{
-    return m_mainWindow;
-}
-
-Settings::PrintDialog& Settings::printDialog()
-{
-    return m_printDialog;
-}
-
 Settings::Settings(QObject* parent) : QObject(parent),
     m_settings(new QSettings("qpdfview", "qpdfview", this)),
     m_pageItem(m_settings),
