@@ -659,6 +659,28 @@ void SettingsDialog::resetGraphicsTab()
 
 void SettingsDialog::createInterfaceTab()
 {
+    // extended search dock
+
+    m_extendedSearchDock = new QCheckBox(this);
+    m_extendedSearchDock->setChecked(s_settings->mainWindow().extendedSearchDock());
+    m_extendedSearchDock->setToolTip(tr("Effective after restart."));
+
+    m_interfaceLayout->addRow(tr("Extended search dock:"), m_extendedSearchDock);
+
+    // annotation overlay
+
+    m_annotationOverlayCheckBox = new QCheckBox(this);
+    m_annotationOverlayCheckBox->setChecked(s_settings->pageItem().annotationOverlay());
+
+    m_interfaceLayout->addRow(tr("Annotation overlay:"), m_annotationOverlayCheckBox);
+
+    // form field overlay
+
+    m_formFieldOverlayCheckBox = new QCheckBox(this);
+    m_formFieldOverlayCheckBox->setChecked(s_settings->pageItem().formFieldOverlay());
+
+    m_interfaceLayout->addRow(tr("Form field overlay:"), m_formFieldOverlayCheckBox);
+
     // tab position
 
     m_tabPositionComboBox = new QComboBox(this);
@@ -694,6 +716,13 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("New tab next to current tab:"), m_newTabNextToCurrentTabCheckBox);
 
+    // exit after last tab
+
+    m_exitAfterLastTabCheckBox = new QCheckBox(this);
+    m_exitAfterLastTabCheckBox->setChecked(s_settings->mainWindow().exitAfterLastTab());
+
+    m_interfaceLayout->addRow(tr("Exit after last tab:"), m_exitAfterLastTabCheckBox);
+
     // recently used count
 
     m_recentlyUsedCountSpinBox = new QSpinBox(this);
@@ -712,13 +741,6 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Recently closed count:"), m_recentlyClosedCountSpinBox);
 
-    // toggle tool and menu bars with fullscreen
-
-    m_toggleToolAndMenuBarsWithFullscreenCheckBox = new QCheckBox(this);
-    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(s_settings->mainWindow().toggleToolAndMenuBarsWithFullscreen());
-
-    m_interfaceLayout->addRow(tr("Toggle tool and menu bars with fullscreen:"), m_toggleToolAndMenuBarsWithFullscreenCheckBox);
-
     // file tool bar
 
     m_fileToolBarLineEdit = new QLineEdit(this);
@@ -743,6 +765,28 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("View tool bar:"), m_viewToolBarLineEdit);
 
+    // scrollable menus
+
+    m_scrollableMenusCheckBox = new QCheckBox(this);
+    m_scrollableMenusCheckBox->setChecked(s_settings->mainWindow().scrollableMenus());
+    m_scrollableMenusCheckBox->setToolTip(tr("Effective after restart."));
+
+    m_interfaceLayout->addRow(tr("Scrollable menus:"), m_scrollableMenusCheckBox);
+
+    // searchable menus
+
+    m_searchableMenusCheckBox = new QCheckBox(this);
+    m_searchableMenusCheckBox->setChecked(s_settings->mainWindow().searchableMenus());
+
+    m_interfaceLayout->addRow(tr("Searchable menus:"), m_searchableMenusCheckBox);
+
+    // toggle tool and menu bars with fullscreen
+
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox = new QCheckBox(this);
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(s_settings->mainWindow().toggleToolAndMenuBarsWithFullscreen());
+
+    m_interfaceLayout->addRow(tr("Toggle tool and menu bars with fullscreen:"), m_toggleToolAndMenuBarsWithFullscreenCheckBox);
+
     // use page label
 
     m_usePageLabelCheckBox = new QCheckBox(this);
@@ -784,39 +828,34 @@ void SettingsDialog::createInterfaceTab()
     m_limitThumbnailsToResultsCheckBox->setChecked(s_settings->documentView().limitThumbnailsToResults());
 
     m_interfaceLayout->addRow(tr("Limit thumbnails to results:"), m_limitThumbnailsToResultsCheckBox);
-
-    // annotation overlay
-
-    m_annotationOverlayCheckBox = new QCheckBox(this);
-    m_annotationOverlayCheckBox->setChecked(s_settings->pageItem().annotationOverlay());
-
-    m_interfaceLayout->addRow(tr("Annotation overlay:"), m_annotationOverlayCheckBox);
-
-    // form field overlay
-
-    m_formFieldOverlayCheckBox = new QCheckBox(this);
-    m_formFieldOverlayCheckBox->setChecked(s_settings->pageItem().formFieldOverlay());
-
-    m_interfaceLayout->addRow(tr("Form field overlay:"), m_formFieldOverlayCheckBox);
 }
 
 void SettingsDialog::acceptInterfaceTab()
 {
+    s_settings->mainWindow().setExtendedSearchDock(m_extendedSearchDock->isChecked());
+
+    s_settings->pageItem().setAnnotationOverlay(m_annotationOverlayCheckBox->isChecked());
+    s_settings->pageItem().setFormFieldOverlay(m_formFieldOverlayCheckBox);
+
     s_settings->mainWindow().setTabPosition(m_tabPositionComboBox->itemData(m_tabPositionComboBox->currentIndex()).toInt());
     s_settings->mainWindow().setTabVisibility(m_tabVisibilityComboBox->itemData(m_tabVisibilityComboBox->currentIndex()).toInt());
     s_settings->mainWindow().setSpreadTabs(m_spreadTabsCheckBox->isChecked());
 
     s_settings->mainWindow().setNewTabNextToCurrentTab(m_newTabNextToCurrentTabCheckBox->isChecked());
+    s_settings->mainWindow().setExitAfterLastTab(m_exitAfterLastTabCheckBox->isChecked());
 
     s_settings->mainWindow().setRecentlyUsedCount(m_recentlyUsedCountSpinBox->value());
     s_settings->mainWindow().setRecentlyClosedCount(m_recentlyClosedCountSpinBox->value());
 
-    s_settings->mainWindow().setToggleToolAndMenuBarsWithFullscreen(m_toggleToolAndMenuBarsWithFullscreenCheckBox->isChecked());
-
     s_settings->mainWindow().setFileToolBar(m_fileToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setEditToolBar(m_editToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setViewToolBar(m_viewToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
 
+    s_settings->mainWindow().setScrollableMenus(m_scrollableMenusCheckBox->isChecked());
+    s_settings->mainWindow().setSearchableMenus(m_searchableMenusCheckBox->isChecked());
+
+    s_settings->mainWindow().setToggleToolAndMenuBarsWithFullscreen(m_toggleToolAndMenuBarsWithFullscreenCheckBox->isChecked());
+
     s_settings->mainWindow().setUsePageLabel(m_usePageLabelCheckBox->isChecked());
     s_settings->mainWindow().setDocumentTitleAsTabTitle(m_documentTitleAsTabTitleCheckBox->isChecked());
 
@@ -825,27 +864,34 @@ void SettingsDialog::acceptInterfaceTab()
 
     s_settings->documentView().setHighlightCurrentThumbnail(m_highlightCurrentThumbnailCheckBox->isChecked());
     s_settings->documentView().setLimitThumbnailsToResults(m_limitThumbnailsToResultsCheckBox->isChecked());
-
-    s_settings->pageItem().setAnnotationOverlay(m_annotationOverlayCheckBox->isChecked());
-    s_settings->pageItem().setFormFieldOverlay(m_formFieldOverlayCheckBox);
 }
 
 void SettingsDialog::resetInterfaceTab()
 {
+    m_extendedSearchDock->setChecked(Defaults::MainWindow::extendedSearchDock());
+
+    m_annotationOverlayCheckBox->setChecked(Defaults::PageItem::annotationOverlay());
+    m_formFieldOverlayCheckBox->setChecked(Defaults::PageItem::formFieldOverlay());
+
     m_tabPositionComboBox->setCurrentIndex(m_tabPositionComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabPosition())));
     m_tabVisibilityComboBox->setCurrentIndex(m_tabVisibilityComboBox->findData(static_cast< uint >(Defaults::MainWindow::tabVisibility())));
     m_spreadTabsCheckBox->setChecked(Defaults::MainWindow::spreadTabs());
 
     m_newTabNextToCurrentTabCheckBox->setChecked(Defaults::MainWindow::newTabNextToCurrentTab());
+    m_exitAfterLastTabCheckBox->setChecked(Defaults::MainWindow::exitAfterLastTab());
 
     m_recentlyUsedCountSpinBox->setValue(Defaults::MainWindow::recentlyUsedCount());
-
-    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(Defaults::MainWindow::toggleToolAndMenuBarsWithFullscreen());
+    m_recentlyClosedCountSpinBox->setValue(Defaults::MainWindow::recentlyClosedCount());
 
     m_fileToolBarLineEdit->setText(Defaults::MainWindow::fileToolBar().join(","));
     m_editToolBarLineEdit->setText(Defaults::MainWindow::editToolBar().join(","));
     m_viewToolBarLineEdit->setText(Defaults::MainWindow::viewToolBar().join(","));
 
+    m_scrollableMenusCheckBox->setChecked(Defaults::MainWindow::scrollableMenus());
+    m_searchableMenusCheckBox->setChecked(Defaults::MainWindow::searchableMenus());
+
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(Defaults::MainWindow::toggleToolAndMenuBarsWithFullscreen());
+
     m_usePageLabelCheckBox->setChecked(Defaults::MainWindow::usePageLabel());
     m_documentTitleAsTabTitleCheckBox->setChecked(Defaults::MainWindow::documentTitleAsTabTitle());
 
@@ -854,9 +900,6 @@ void SettingsDialog::resetInterfaceTab()
 
     m_highlightCurrentThumbnailCheckBox->setChecked(Defaults::DocumentView::highlightCurrentThumbnail());
     m_limitThumbnailsToResultsCheckBox->setChecked(Defaults::DocumentView::limitThumbnailsToResults());
-
-    m_annotationOverlayCheckBox->setChecked(Defaults::PageItem::annotationOverlay());
-    m_formFieldOverlayCheckBox->setChecked(Defaults::PageItem::formFieldOverlay());
 }
 
 void SettingsDialog::createModifiersTab()
@@ -890,6 +933,12 @@ void SettingsDialog::createModifiersTab()
     createModifiersComboBox(m_addAnnotationModifiersComboBox, s_settings->pageItem().addAnnotationModifiers());
 
     m_modifiersLayout->addRow(tr("Add annotation:"), m_addAnnotationModifiersComboBox);
+
+    // zoom to selection modifiers
+
+    createModifiersComboBox(m_zoomToSelectionModifiersComboBox, s_settings->pageItem().zoomToSelectionModifiers());
+
+    m_modifiersLayout->addRow(tr("Zoom to selection:"), m_zoomToSelectionModifiersComboBox);
 }
 
 void SettingsDialog::acceptModifiersTab()
@@ -900,6 +949,7 @@ void SettingsDialog::acceptModifiersTab()
 
     s_settings->pageItem().setCopyToClipboardModifiers(getKeyboardModifierFromItemData(m_copyToClipboardModifiersComboBox));
     s_settings->pageItem().setAddAnnotationModifiers(getKeyboardModifierFromItemData(m_addAnnotationModifiersComboBox));
+    s_settings->pageItem().setZoomToSelectionModifiers(getKeyboardModifierFromItemData(m_zoomToSelectionModifiersComboBox));
 }
 
 void SettingsDialog::resetModifiersTab()
@@ -910,6 +960,7 @@ void SettingsDialog::resetModifiersTab()
 
     setCurrentIndexFromKeyboardModifiers(m_copyToClipboardModifiersComboBox, Defaults::PageItem::copyToClipboardModifiers());
     setCurrentIndexFromKeyboardModifiers(m_addAnnotationModifiersComboBox, Defaults::PageItem::addAnnotationModifiers());
+    setCurrentIndexFromKeyboardModifiers(m_zoomToSelectionModifiersComboBox, Defaults::PageItem::zoomToSelectionModifiers());
 }
 
 void SettingsDialog::createColorComboBox(QComboBox*& comboBox, const QColor& color)
@@ -932,6 +983,7 @@ void SettingsDialog::createModifiersComboBox(QComboBox*& comboBox, const Qt::Key
     comboBox->addItem(QShortcut::tr("Shift and Alt"), static_cast< int >(Qt::ShiftModifier | Qt::AltModifier));
     comboBox->addItem(QShortcut::tr("Ctrl and Alt"), static_cast< int >(Qt::ControlModifier | Qt::AltModifier));
     comboBox->addItem(QShortcut::tr("Right mouse button"), static_cast< int >(Qt::RightButton));
+    comboBox->addItem(QShortcut::tr("Middle mouse button"), static_cast< int >(Qt::MidButton));
 
     setCurrentIndexFromKeyboardModifiers(comboBox, modifiers);
 }
