@@ -146,21 +146,30 @@ private:
 
     // interface
 
+    QCheckBox* m_extendedSearchDock;
+
+    QCheckBox* m_annotationOverlayCheckBox;
+    QCheckBox* m_formFieldOverlayCheckBox;
+
     QComboBox* m_tabPositionComboBox;
     QComboBox* m_tabVisibilityComboBox;
     QCheckBox* m_spreadTabsCheckBox;
 
     QCheckBox* m_newTabNextToCurrentTabCheckBox;
+    QCheckBox* m_exitAfterLastTabCheckBox;
 
     QSpinBox* m_recentlyUsedCountSpinBox;
     QSpinBox* m_recentlyClosedCountSpinBox;
 
-    QCheckBox* m_toggleToolAndMenuBarsWithFullscreenCheckBox;
-
     QLineEdit* m_fileToolBarLineEdit;
     QLineEdit* m_editToolBarLineEdit;
     QLineEdit* m_viewToolBarLineEdit;
 
+    QCheckBox* m_scrollableMenusCheckBox;
+    QCheckBox* m_searchableMenusCheckBox;
+
+    QCheckBox* m_toggleToolAndMenuBarsWithFullscreenCheckBox;
+
     QCheckBox* m_usePageLabelCheckBox;
     QCheckBox* m_documentTitleAsTabTitleCheckBox;
 
@@ -170,9 +179,6 @@ private:
     QCheckBox* m_highlightCurrentThumbnailCheckBox;
     QCheckBox* m_limitThumbnailsToResultsCheckBox;
 
-    QCheckBox* m_annotationOverlayCheckBox;
-    QCheckBox* m_formFieldOverlayCheckBox;
-
     void createInterfaceTab();
     void acceptInterfaceTab();
     void resetInterfaceTab();
@@ -185,6 +191,7 @@ private:
 
     QComboBox* m_copyToClipboardModifiersComboBox;
     QComboBox* m_addAnnotationModifiersComboBox;
+    QComboBox* m_zoomToSelectionModifiersComboBox;
 
     void createModifiersTab();
     void acceptModifiersTab();
