@@ -28,6 +28,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 class QSettings;
 
+namespace qpdfview
+{
+
 class ShortcutHandler : public QAbstractTableModel
 {
     Q_OBJECT
@@ -85,4 +88,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // SHORTCUTHANDLER_H
